#include "../pluginsdk/bridgemain.h"
#include "../pluginsdk/_plugins.h"
#include "../pluginsdk/_dbgfunctions.h"
#include "../pluginsdk/_scriptapi_function.h"
#include "../pluginsdk/_scriptapi_module.h"
#include <string>
#include <vector>
#include <map>
#include <set>
#include <fstream>
#include <sstream>
#include <iostream>
#include <iomanip>
#include <ctime>
#include <algorithm>
#include <Windows.h>
#include <Psapi.h> // For MODULEINFO

// Базовая информация о плагине
#define PLUGIN_NAME "Funcap"
#define PLUGIN_VERSION 1

// Define constants needed for our code
#define STRING_EXPLORATION_MIN_LENGTH 4
#define STRING_LENGTH 164
#define STRING_LENGTH_IN_COMMENTS 82
#define CALL_COLOR 0x99CCCC
#define FUNC_COLOR 0xF7CBEA
#define ITEM_COLOR 0x70E01B
#define MAX_ARGS 10
#define DEFAULT_LOG_FILENAME "funcap_log.txt"
#define DEFAULT_BLACKLIST_FILENAME "funcap_blacklist.txt"
#define MENU_AUTO_RUN 7
#define MENU_PAUSE_AUTO_RUN 8
#define MENU_MANAGE_BLACKLIST 9
#define MENU_TEST_READ_STRINGS 10 // Добавляем новый идентификатор меню
#define DEFAULT_STRINGS_FILENAME "funcap_strings.txt"

// Определения инструкций
#define TYPE_CALL 3
#define TYPE_RET 4

// Структура для хранения информации о вызове функции
struct FunctionCall
{
    duint callerAddr;
    duint functionAddr;
    duint returnAddr;
    std::string functionName;
    std::vector<duint> arguments;
    std::map<std::string, duint> registers;
    duint returnValue;
    bool processed;
};

// Структура для хранения контекста
struct RegisterContext
{
    std::string name;
    duint value;
};

// Глобальные переменные
int pluginHandle;
HWND hwndDlg;
int hMenu;
int hFuncapMenu;
bool pluginRunning = false;
std::ofstream logFile;
std::ofstream stringsFile; // Добавляем файл для сохранения строк
std::string logFilePath;
std::string blacklistFilePath;
std::string stringsFilePath; // Добавляем путь к файлу строк
bool deleteBreakpoints = true;
bool addComments = true;
bool hexDumpMode = false;
bool colorEnable = true;
bool isX64 = false;
std::map<duint, FunctionCall> activeFunctionCalls;
std::map<duint, duint> breakpointsMap; // Map addresses to their original bytes
std::vector<duint> visitedAddresses;
DBGFUNCTIONS dbgFunctions;
bool autoRunMode = false;
std::set<std::string> blacklistedModules; // Set to store blacklisted module names

// Прототипы функций
bool pluginInit(PLUG_INITSTRUCT *initStruct);
void pluginStop();
void pluginSetup();
bool cbBreakpoint(CBTYPE cbType, PLUG_CB_BREAKPOINT *bp);
bool cbSystemBreakpoint(CBTYPE cbType, PLUG_CB_SYSTEMBREAKPOINT *info);
bool cbDebugEvent(CBTYPE cbType, PLUG_CB_DEBUGEVENT *info);
bool cbMenuEntry(CBTYPE cbType, PLUG_CB_MENUENTRY *info);
void menuAddFuncStart();
void menuAddFuncRet();
void menuAddCallInstructions();
void menuDeleteAllBreakpoints();
void menuToggleLogging();
void menuAutoRunToEnd();
void menuPauseAutoRun();
void menuManageBlacklist();
void menuTestReadStrings(); // Добавляем новую функцию для тестирования чтения строк
bool isModuleBlacklisted(const char *moduleName);
void loadBlacklistFromFile();
void saveBlacklistToFile();
std::vector<RegisterContext> getRegisters();
bool isCallInstruction(duint addr);
bool isRetInstruction(duint addr);
duint getReturnAddress();
std::string getAsciiString(const std::vector<unsigned char> &data);
std::string getPrintableString(const std::vector<unsigned char> &data, bool printDots = true);
std::string smartFormat(duint address, int maxLen);
std::string getCurrentTime();
void addLogLine(const std::string &line);
void handleFunctionCall(duint addr);
void handleFunctionReturn(duint addr);
bool getFunctionNameAtAddress(duint addr, char *name, int maxNameSize);
duint findFunctionStart(duint addr);
std::vector<unsigned char> readMemory(duint address, size_t size);
std::string analyzeStringData(duint address);
std::string formatDetectedStrings(const std::vector<RegisterContext> &registers, const std::vector<duint> &args);
bool isPotentialStringLocation(duint addr);
void setBreakpointsAtStringLocations(duint moduleBase, size_t moduleSize);
bool GetModuleSize(duint moduleBase, duint *moduleSize);

// Функция для гарантированной записи строки в файл строк
bool writeToStringsFile(const std::string &str, const std::string &context = "")
{
    static int fileErrorCounter = 0; // Счетчик ошибок файла
    const int MAX_ERRORS = 5;        // Максимальное количество ошибок перед принудительным переоткрытием
    static CRITICAL_SECTION csStringsFile;
    static bool csInitialized = false;

    if (!csInitialized)
    {
        InitializeCriticalSection(&csStringsFile);
        csInitialized = true;
    }

    // Используем критическую секцию для безопасного доступа к файлу
    EnterCriticalSection(&csStringsFile);
    bool success = false;

    try
    {
        // Если файл не открыт, пытаемся открыть его
        if (!stringsFile.is_open())
        {
            _plugin_logprintf("[" PLUGIN_NAME "] Strings file not open, trying to open: %s\n", stringsFilePath.c_str());
            stringsFile.open(stringsFilePath, std::ios::out | std::ios::app);

            if (!stringsFile.is_open())
            {
                _plugin_logprintf("[" PLUGIN_NAME "] Failed to open strings file: %s\n", stringsFilePath.c_str());

                // Пробуем создать файл в директории с плагином, если текущий путь недоступен
                char currentDir[MAX_PATH] = {0};
                GetCurrentDirectoryA(MAX_PATH, currentDir);
                std::string localStringsPath = std::string(currentDir) + "\\" + DEFAULT_STRINGS_FILENAME;

                _plugin_logprintf("[" PLUGIN_NAME "] Trying to create strings file in plugin directory: %s\n", localStringsPath.c_str());
                stringsFile.open(localStringsPath, std::ios::out | std::ios::app);

                if (stringsFile.is_open())
                {
                    // Если удалось открыть файл в новом месте, обновляем путь
                    stringsFilePath = localStringsPath;
                    _plugin_logprintf("[" PLUGIN_NAME "] Successfully created strings file in: %s\n", stringsFilePath.c_str());
                }
                else
                {
                    // Если все попытки открыть файл не удались, пытаемся использовать системную временную директорию
                    char tempPath[MAX_PATH] = {0};
                    GetTempPathA(MAX_PATH, tempPath);
                    std::string tempStringsPath = std::string(tempPath) + DEFAULT_STRINGS_FILENAME;

                    _plugin_logprintf("[" PLUGIN_NAME "] Trying to create strings file in temp directory: %s\n", tempStringsPath.c_str());
                    stringsFile.open(tempStringsPath, std::ios::out | std::ios::app);

                    if (stringsFile.is_open())
                    {
                        // Обновляем путь при успехе
                        stringsFilePath = tempStringsPath;
                        _plugin_logprintf("[" PLUGIN_NAME "] Successfully created strings file in temp directory: %s\n", stringsFilePath.c_str());
                    }
                    else
                    {
                        LeaveCriticalSection(&csStringsFile);
                        _plugin_logprintf("[" PLUGIN_NAME "] Failed to create strings file in any location!\n");
                        return false;
                    }
                }
            }
        }

        // Проверяем состояние файла
        if (stringsFile.fail())
        {
            _plugin_logprintf("[" PLUGIN_NAME "] Strings file in fail state, clearing and reopening\n");
            fileErrorCounter++;

            if (fileErrorCounter >= MAX_ERRORS)
            {
                _plugin_logprintf("[" PLUGIN_NAME "] Too many file errors (%d). Forcing file reopen...\n", fileErrorCounter);
                stringsFile.close();
                stringsFile.open(stringsFilePath, std::ios::out | std::ios::app);
                fileErrorCounter = 0;
            }
            else
            {
                stringsFile.clear();
                if (stringsFile.fail())
                {
                    _plugin_logprintf("[" PLUGIN_NAME "] Still in fail state after clear. Reopening file...\n");
                    stringsFile.close();
                    stringsFile.open(stringsFilePath, std::ios::out | std::ios::app);
                }
            }

            if (!stringsFile.is_open() || stringsFile.fail())
            {
                _plugin_logprintf("[" PLUGIN_NAME "] Could not reset strings file stream\n");
                fileErrorCounter++;
                LeaveCriticalSection(&csStringsFile);
                return false;
            }
        }

        // Форматируем строку с дополнительной информацией
        std::string timeStr = getCurrentTime();
        std::string outputLine;

        if (!context.empty())
        {
            outputLine = "[" + timeStr + "] [" + context + "] " + str;
        }
        else
        {
            outputLine = "[" + timeStr + "] " + str;
        }

        // Записываем строку и сбрасываем буфер
        stringsFile << outputLine << std::endl;
        stringsFile.flush();

        if (stringsFile.fail())
        {
            _plugin_logprintf("[" PLUGIN_NAME "] Failed to write to strings file after write operation\n");
            fileErrorCounter++;
            LeaveCriticalSection(&csStringsFile);
            return false;
        }

        fileErrorCounter = 0; // Сбрасываем счетчик при успешной записи
        success = true;
    }
    catch (const std::exception &e)
    {
        _plugin_logprintf("[" PLUGIN_NAME "] Exception in writeToStringsFile: %s\n", e.what());
        success = false;
    }
    catch (...)
    {
        _plugin_logprintf("[" PLUGIN_NAME "] Unknown exception in writeToStringsFile\n");
        success = false;
    }

    LeaveCriticalSection(&csStringsFile);
    return success;
}

// Add this function to extract and format detected strings for the log
std::string formatDetectedStrings(const std::vector<RegisterContext> &registers, const std::vector<duint> &args)
{
    std::vector<std::pair<std::string, std::string>> detectedStrings;

    // Check registers for strings
    for (const auto &reg : registers)
    {
        duint value = reg.value;
        if (value == 0)
            continue;

        // Не пытаемся извлечь строки из регистров управления, IP и системных регистров
        if (reg.name == "EIP" || reg.name == "RIP" ||
            reg.name == "ESP" || reg.name == "RSP" ||
            reg.name == "EBP" || reg.name == "RBP" ||
            reg.name == "EFLAGS" || reg.name == "RFLAGS" ||
            reg.name.find("DR") == 0 || // Debug registers (DR0-DR7)
            reg.name.find("CR") == 0)   // Control registers (CR0-CR8)
            continue;

        // Проверяем, является ли значение регистра допустимым указателем для чтения
        if (!DbgMemIsValidReadPtr(value))
            continue;

        std::string strData = analyzeStringData(value);
        if (!strData.empty() &&
            (strData[0] == '"' || strData.substr(0, 2) == "L\"")) // Если это действительно строка
        {
            detectedStrings.push_back(std::make_pair(reg.name, strData));

            // Сохраняем строку в файл строк
            try
            {
                // Получаем строку без кавычек для сохранения
                std::string cleanStr = strData;
                if (cleanStr[0] == '"')
                    cleanStr = cleanStr.substr(1, cleanStr.length() - 2);
                else if (cleanStr.substr(0, 2) == "L\"")
                    cleanStr = cleanStr.substr(2, cleanStr.length() - 3);

                _plugin_logprintf("[" PLUGIN_NAME "] Writing string from %s to file: %s\n", reg.name.c_str(), cleanStr.c_str());

                // Используем надежную функцию записи вместо прямого доступа к файлу
                bool writeSuccess = writeToStringsFile(cleanStr, "REG:" + reg.name);
                if (writeSuccess)
                {
                    _plugin_logprintf("[" PLUGIN_NAME "] Successfully wrote string from %s to file\n", reg.name.c_str());
                }
                else
                {
                    _plugin_logprintf("[" PLUGIN_NAME "] Failed to write string from %s to file\n", reg.name.c_str());
                }
            }
            catch (const std::exception &e)
            {
                _plugin_logprintf("[" PLUGIN_NAME "] Error formatting string from %s: %s\n", reg.name.c_str(), e.what());
            }
        }
    }

    // Check arguments for strings
    for (size_t i = 0; i < args.size(); i++)
    {
        duint arg = args[i];
        if (arg == 0)
            continue;

        // Проверяем, является ли аргумент допустимым указателем для чтения
        if (!DbgMemIsValidReadPtr(arg))
            continue;

        std::string strData = analyzeStringData(arg);
        if (!strData.empty() &&
            (strData[0] == '"' || strData.substr(0, 2) == "L\"")) // Если это действительно строка
        {
            detectedStrings.push_back(std::make_pair("arg" + std::to_string(i), strData));

            // Сохраняем строку в файл строк
            try
            {
                // Получаем строку без кавычек для сохранения
                std::string cleanStr = strData;
                if (cleanStr[0] == '"')
                    cleanStr = cleanStr.substr(1, cleanStr.length() - 2);
                else if (cleanStr.substr(0, 2) == "L\"")
                    cleanStr = cleanStr.substr(2, cleanStr.length() - 3);

                _plugin_logprintf("[" PLUGIN_NAME "] Writing string from arg%d to file: %s\n", (int)i, cleanStr.c_str());

                // Используем надежную функцию записи вместо прямого доступа к файлу
                bool writeSuccess = writeToStringsFile(cleanStr, "ARG:" + std::to_string(i));
                if (writeSuccess)
                {
                    _plugin_logprintf("[" PLUGIN_NAME "] Successfully wrote string from arg%d to file\n", (int)i);
                }
                else
                {
                    _plugin_logprintf("[" PLUGIN_NAME "] Failed to write string from arg%d to file\n", (int)i);
                }
            }
            catch (const std::exception &e)
            {
                _plugin_logprintf("[" PLUGIN_NAME "] Error formatting string from arg%d: %s\n", (int)i, e.what());
            }
        }
    }

    // Format the detected strings for output
    if (detectedStrings.empty())
        return "";

    // Подробное логирование найденных строк для отладки
    _plugin_logprintf("[" PLUGIN_NAME "] ==========[ DETECTED STRINGS ]==========\n");
    for (const auto &str : detectedStrings)
    {
        _plugin_logprintf("[" PLUGIN_NAME "] %s = %s\n", str.first.c_str(), str.second.c_str());

        // Добавляем сообщение в GUI лог с более заметным форматированием
        GuiAddLogMessage((">>> [FunCap] STRING FOUND: " + str.first + " = " + str.second + "\n").c_str());
    }
    _plugin_logprintf("[" PLUGIN_NAME "] ======================================\n");

    std::stringstream ss;
    ss << "STRINGS: ";
    for (size_t i = 0; i < detectedStrings.size(); i++)
    {
        if (i > 0)
            ss << ", ";
        ss << detectedStrings[i].first << "=" << detectedStrings[i].second;
    }
    return ss.str();
}

// Конвертировать duint в шестнадцатеричную строку
std::string hexToString(duint value)
{
    std::stringstream ss;
    ss << std::hex << value;
    return "0x" + ss.str();
}

// Получить имя по адресу
std::string getName(duint addr)
{
    char name[MAX_PATH] = "";
    if (getFunctionNameAtAddress(addr, name, MAX_PATH))
        return std::string(name);
    return hexToString(addr);
}

// Проверяет, существует ли файл
bool fileExists(const std::string &filename)
{
    std::ifstream file(filename);
    return file.good();
}

// Функция для экспорта, используемая x64dbg для загрузки плагина
extern "C" __declspec(dllexport) bool pluginit(PLUG_INITSTRUCT *initStruct)
{
    initStruct->pluginVersion = PLUGIN_VERSION;
    initStruct->sdkVersion = PLUG_SDKVERSION;
    strcpy_s(initStruct->pluginName, PLUGIN_NAME);
    pluginHandle = initStruct->pluginHandle;

// Определяем архитектуру
#ifdef _WIN64
    isX64 = true;
#else
    isX64 = false;
#endif

    // Получаем доступ к функциям отладчика
    dbgFunctions = *DbgFunctions();
    if (!dbgFunctions.AssembleAtEx)
    {
        _plugin_logprintf("[" PLUGIN_NAME "] Failed to get debuggee functions!\n");
        return false;
    }

    // Устанавливаем путь к лог-файлу в директории рядом с плагином или исполняемым файлом
    char pluginPath[MAX_PATH] = {0};
    char currentDir[MAX_PATH] = {0};

    // Для лучшей совместимости сохраняем в текущем рабочем каталоге
    GetCurrentDirectoryA(MAX_PATH, currentDir);
    logFilePath = std::string(currentDir) + "\\" + DEFAULT_LOG_FILENAME;
    stringsFilePath = std::string(currentDir) + "\\" + DEFAULT_STRINGS_FILENAME; // Добавляем путь к файлу строк

    // Initialize blacklist
    loadBlacklistFromFile();

    // Проверяем доступность для записи
    std::ofstream testFile(logFilePath.c_str(), std::ios::out);
    if (testFile.is_open())
    {
        testFile << "FunCap Log File - Created at " << getCurrentTime() << std::endl;
        testFile.close();
        _plugin_logprintf("[" PLUGIN_NAME "] Log file location: %s\n", logFilePath.c_str());

        // Также проверяем файл строк
        std::ofstream testStringsFile(stringsFilePath.c_str(), std::ios::out | std::ios::app);
        if (testStringsFile.is_open())
        {
            if (!fileExists(stringsFilePath))
            {
                testStringsFile << "FunCap Strings File - Created at " << getCurrentTime() << std::endl;
            }
            testStringsFile.close();
            _plugin_logprintf("[" PLUGIN_NAME "] Strings file location: %s\n", stringsFilePath.c_str());
        }
        else
        {
            _plugin_logprintf("[" PLUGIN_NAME "] WARNING: Could not create strings file. String capturing may not work!\n");
        }
    }
    else
    {
        // В случае ошибки пробуем записать в TEMP директорию
        char tempPath[MAX_PATH] = {0};
        GetTempPathA(MAX_PATH, tempPath);
        logFilePath = std::string(tempPath) + DEFAULT_LOG_FILENAME;
        stringsFilePath = std::string(tempPath) + DEFAULT_STRINGS_FILENAME; // Добавляем путь к файлу строк в TEMP

        _plugin_logprintf("[" PLUGIN_NAME "] Falling back to TEMP directory for log file: %s\n", logFilePath.c_str());

        // Проверяем доступность и для этого пути
        testFile.open(logFilePath.c_str(), std::ios::out);
        if (testFile.is_open())
        {
            testFile << "FunCap Log File - Created at " << getCurrentTime() << std::endl;
            testFile.close();
        }
        else
        {
            _plugin_logprintf("[" PLUGIN_NAME "] WARNING: Could not create log file. Logging may not work!\n");
        }
    }

    // Регистрируем колбэки
    _plugin_registercallback(pluginHandle, CB_BREAKPOINT, (CBPLUGIN)cbBreakpoint);
    _plugin_registercallback(pluginHandle, CB_SYSTEMBREAKPOINT, (CBPLUGIN)cbSystemBreakpoint);
    _plugin_registercallback(pluginHandle, CB_DEBUGEVENT, (CBPLUGIN)cbDebugEvent);
    _plugin_registercallback(pluginHandle, CB_MENUENTRY, (CBPLUGIN)cbMenuEntry);

    return true;
}

// Вызывается при окончании работы плагина
extern "C" __declspec(dllexport) void plugstop()
{
    pluginStop();
}

// Вызывается после инициализации графического интерфейса
extern "C" __declspec(dllexport) void plugsetup(PLUG_SETUPSTRUCT *setupStruct)
{
    hwndDlg = setupStruct->hwndDlg;
    hMenu = setupStruct->hMenu;

    // Создаем подменю для плагина
    hFuncapMenu = _plugin_menuadd(hMenu, "&Funcap");

    // Добавляем элементы в подменю
    _plugin_menuaddentry(hFuncapMenu, 0, "▶ Start Capture");
    _plugin_menuaddentry(hFuncapMenu, 1, "⏹ Stop Capture");
    _plugin_menuaddentry(hFuncapMenu, 2, "🔍 Add Breakpoints on All Functions");
    _plugin_menuaddentry(hFuncapMenu, 3, "↩ Add Breakpoints on All RET Instructions");
    _plugin_menuaddentry(hFuncapMenu, 4, "➡ Add Breakpoints on All CALL Instructions");
    _plugin_menuaddentry(hFuncapMenu, 5, "❌ Delete All Breakpoints");
    _plugin_menuaddentry(hFuncapMenu, MENU_AUTO_RUN, "⏵ &Run Automatic To End And Save");
    _plugin_menuaddentry(hFuncapMenu, MENU_PAUSE_AUTO_RUN, "⏸ &Pause Auto Run");
    _plugin_menuaddentry(hFuncapMenu, MENU_MANAGE_BLACKLIST, "📋 Manage Module &Blacklist");
    _plugin_menuaddentry(hFuncapMenu, MENU_TEST_READ_STRINGS, "📝 Test &Read Strings from Registers"); // Новый пункт меню

    pluginSetup();
}

// Инициализация плагина
bool pluginInit(PLUG_INITSTRUCT *initStruct)
{
    return true;
}

// Остановка плагина
void pluginStop()
{
    if (logFile.is_open())
    {
        try
        {
            logFile << "=== FunCap Plugin Stopped at " << getCurrentTime() << " ===" << std::endl;
            logFile.flush();
            logFile.close();
            _plugin_logprintf("[" PLUGIN_NAME "] Log file closed\n");
        }
        catch (const std::exception &e)
        {
            _plugin_logprintf("[" PLUGIN_NAME "] Error closing log file: %s\n", e.what());
        }
    }

    // Закрываем файл строк, если он открыт
    if (stringsFile.is_open())
    {
        try
        {
            stringsFile.flush();
            stringsFile.close();
            _plugin_logprintf("[" PLUGIN_NAME "] Strings file closed\n");
        }
        catch (const std::exception &e)
        {
            _plugin_logprintf("[" PLUGIN_NAME "] Error closing strings file: %s\n", e.what());
        }
    }

    // Сохраняем черный список перед завершением
    _plugin_logprintf("[" PLUGIN_NAME "] Saving module blacklist with %d entries\n", (int)blacklistedModules.size());
    saveBlacklistToFile();

    pluginRunning = false;

    // Очищаем сохраненные данные
    activeFunctionCalls.clear();
    breakpointsMap.clear();
    visitedAddresses.clear();

    _plugin_logprintf("[" PLUGIN_NAME "] Plugin stopped\n");
}

// Настройка плагина
void pluginSetup()
{
    // Настройка UI и другие инициализации после загрузки GUI
    pluginRunning = false;
}

// Обработчик точек останова
bool cbBreakpoint(CBTYPE cbType, PLUG_CB_BREAKPOINT *bp)
{
    if (bp->breakpoint->addr == 0)
        return false;

    try
    {
        // Обрабатываем точку останова
        _plugin_logprintf("[" PLUGIN_NAME "] Breakpoint hit at address: 0x%p\n", (void *)bp->breakpoint->addr);

        // Проверяем тип точки останова
        if (bp->breakpoint->type == bp_memory)
        {
            // Это точка останова на доступ к памяти, вероятно к строке
            _plugin_logprintf("[" PLUGIN_NAME "] Memory access breakpoint hit at: 0x%p\n", (void *)bp->breakpoint->addr);

            // Сначала удаляем точку останова перед дальнейшей обработкой, чтобы избежать зацикливания
            bool bpDeleted = false;
            if (autoRunMode || deleteBreakpoints)
            {
                char bpCmd[64] = {0};
                sprintf_s(bpCmd, "bpmc %p", (void *)bp->breakpoint->addr);
                _plugin_logprintf("[" PLUGIN_NAME "] Deleting memory breakpoint: %s\n", bpCmd);
                DbgCmdExec(bpCmd);

                // Проверяем, действительно ли удалена точка останова
                Sleep(10); // Небольшая задержка для применения команды
                BPXTYPE bpType = DbgGetBpxTypeAt(bp->breakpoint->addr);
                bpDeleted = !(bpType & bp_memory);

                if (!bpDeleted) {
                    _plugin_logprintf("[" PLUGIN_NAME "] WARNING: Memory breakpoint deletion may have failed. Trying alternative method.\n");
                    // Альтернативный метод удаления
                    sprintf_s(bpCmd, "bc %p", (void *)bp->breakpoint->addr);
                    DbgCmdExec(bpCmd);

                    // Дополнительная проверка
                    Sleep(10);
                    bpType = DbgGetBpxTypeAt(bp->breakpoint->addr);
                    bpDeleted = !(bpType & bp_memory);

                    if (!bpDeleted) {
                        _plugin_logprintf("[" PLUGIN_NAME "] CRITICAL: Failed to delete memory breakpoint at 0x%p\n", (void *)bp->breakpoint->addr);
                    }
                }
            }

            // Получаем данные из этого адреса и проверяем, содержит ли он строку
            try {
                std::string strData = analyzeStringData(bp->breakpoint->addr);
                if (!strData.empty() && (strData[0] == '"' || strData.substr(0, 2) == "L\""))
                {
                    std::string logMsg = "String accessed at 0x" + hexToString(bp->breakpoint->addr) + ": " + strData;
                    _plugin_logprintf("[" PLUGIN_NAME "] %s\n", logMsg.c_str());
                    GuiAddLogMessage(("[FunCap] " + logMsg + "\n").c_str());

                    // Записываем строку в файл, обрабатывая возможные ошибки
                    try {
                        // Записываем строку в файл
                        writeToStringsFile(strData[0] == '"' ? strData.substr(1, strData.length() - 2) : strData.substr(2, strData.length() - 3),
                                       "ACCESS:0x" + hexToString(bp->breakpoint->addr));

                        // Проверяем стек для поиска возможного контекста использования строки
                        duint csp = 0;
                        csp = DbgValFromString("csp");
                        if (csp != 0)
                        {
                            for (int i = 0; i < 10; i++)  // Проверяем первые 10 значений на стеке
                            {
                                duint stackValue = 0;
                                if (DbgMemRead(csp + i * sizeof(duint), &stackValue, sizeof(duint)))
                                {
                                    // Если значение на стеке - адрес функции, получаем её имя
                                    char funcName[MAX_LABEL_SIZE] = {0};
                                    if (DbgGetLabelAt(stackValue, SEG_DEFAULT, funcName))
                                    {
                                        std::string context = "Called from " + std::string(funcName);
                                        _plugin_logprintf("[" PLUGIN_NAME "] %s\n", context.c_str());
                                        GuiAddLogMessage(("[FunCap] " + context + "\n").c_str());
                                        break;
                                    }
                                }
                            }
                        }
                    }
                    catch (const std::exception &e) {
                        _plugin_logprintf("[" PLUGIN_NAME "] Exception while processing string data: %s\n", e.what());
                    }
                    catch (...) {
                        _plugin_logprintf("[" PLUGIN_NAME "] Unknown exception while processing string data\n");
                    }
                }
            }
            catch (const std::exception &e) {
                _plugin_logprintf("[" PLUGIN_NAME "] Exception in analyzeStringData: %s\n", e.what());
            }
            catch (...) {
                _plugin_logprintf("[" PLUGIN_NAME "] Unknown exception in analyzeStringData\n");
            }

            // Если мы в автоматическом режиме, продолжаем выполнение
            if (autoRunMode)
            {
                _plugin_logprintf("[" PLUGIN_NAME "] Auto-continuing after memory breakpoint\n");
                // Обеспечиваем немедленное продолжение выполнения
                DbgCmdExec("run");
                return true; // Выходим из функции, т.к. выполнение уже продолжено
            }
        }
        // Проверяем, является ли это возвратом из функции
        else if (isRetInstruction(bp->breakpoint->addr))
        {
            // Перед обработкой возврата, явно проверяем регистры на строки
            if (autoRunMode)
            {
                _plugin_logprintf("[" PLUGIN_NAME "] Auto-run: checking registers for strings at RET 0x%p\n", (void *)bp->breakpoint->addr);
                std::vector<RegisterContext> registers = getRegisters();
                std::vector<duint> emptyArgs; // На возврате нас интересуют только регистры
                std::string stringDetections = formatDetectedStrings(registers, emptyArgs);

                if (!stringDetections.empty())
                {
                    _plugin_logprintf("[" PLUGIN_NAME "] Strings found at RET: %s\n", stringDetections.c_str());
                    // Дополнительно записываем в файл строк с контекстом RET
                    // writeToStringsFile("Found at RET instruction at " + hexToString(bp->breakpoint->addr), "AUTO-RUN:RET");
                }

                // Проверяем возвращаемое значение на указатель на строку
                duint eax = 0;
                if ((eax = DbgValFromString("eax")) != 0 || (eax = DbgValFromString("rax")) != 0)
                {
                    if (DbgMemIsValidReadPtr(eax))
                    {
                        std::string strData = analyzeStringData(eax);
                        if (!strData.empty() && (strData[0] == '"' || strData.substr(0, 2) == "L\""))
                        {
                            std::string logMsg = "String returned in EAX/RAX: " + strData;
                            _plugin_logprintf("[" PLUGIN_NAME "] %s\n", logMsg.c_str());
                            GuiAddLogMessage(("[FunCap] " + logMsg + "\n").c_str());

                            // Записываем строку в файл
                            writeToStringsFile(strData[0] == '"' ? strData.substr(1, strData.length() - 2) : strData.substr(2, strData.length() - 3),
                                               "RETURN:EAX/RAX");
                        }
                    }
                }
            }

            handleFunctionReturn(bp->breakpoint->addr);

            // Удаляем точку останова, если нужно
            if (deleteBreakpoints)
            {
                _plugin_logprintf("[" PLUGIN_NAME "] Deleting RET breakpoint at: 0x%p\n", (void *)bp->breakpoint->addr);

                // Skip system entry point breakpoints, they're managed by x64dbg
                if (bp->breakpoint->type == bp_normal && strstr(bp->breakpoint->mod, "entry") != NULL)
                {
                    _plugin_logprintf("[" PLUGIN_NAME "] Skipping deletion of entry point breakpoint\n");
                }
                else
                {
                    // Try different approaches for deleting breakpoints
                    char bpCmd[64] = {0};
                    sprintf_s(bpCmd, "bc %p", (void *)bp->breakpoint->addr);
                    _plugin_logprintf("[" PLUGIN_NAME "] Trying command: %s\n", bpCmd);
                    DbgCmdExec(bpCmd);

                    // For hardware breakpoints, try using bphc with the address (not handle)
                    if (bp->breakpoint->type != bp_normal)
                    {
                        sprintf_s(bpCmd, "bphc %p", (void *)bp->breakpoint->addr);
                        _plugin_logprintf("[" PLUGIN_NAME "] Trying hardware breakpoint delete command: %s\n", bpCmd);
                        DbgCmdExec(bpCmd);
                    }
                }
            }
        }
        else
        {
            // Перед обработкой вызова, явно проверяем регистры на строки
            if (autoRunMode)
            {
                _plugin_logprintf("[" PLUGIN_NAME "] Auto-run: checking registers for strings at CALL 0x%p\n", (void *)bp->breakpoint->addr);
                std::vector<RegisterContext> registers = getRegisters();

                // Create empty args vector instead of calling getArguments
                std::vector<duint> args;
                // Get basic register values that might contain arguments
                bool success = false;
                if (isX64)
                {
                    // In x64, common argument registers
                    duint rcx = DbgEval("rcx", &success);
                    if (success)
                        args.push_back(rcx);
                    duint rdx = DbgEval("rdx", &success);
                    if (success)
                        args.push_back(rdx);
                    duint r8 = DbgEval("r8", &success);
                    if (success)
                        args.push_back(r8);
                    duint r9 = DbgEval("r9", &success);
                    if (success)
                        args.push_back(r9);
                }
                else
                {
                    // In x86, try to read stack values
                    duint esp = DbgEval("esp", &success);
                    if (success)
                    {
                        for (int i = 0; i < 4; i++)
                        { // Just check first 4 potential args
                            duint arg = 0;
                            if (DbgMemRead(esp + 4 + i * 4, &arg, 4))
                            {
                                args.push_back(arg);
                            }
                        }
                    }
                }

                std::string stringDetections = formatDetectedStrings(registers, args);

                if (!stringDetections.empty())
                {
                    _plugin_logprintf("[" PLUGIN_NAME "] Strings found at CALL: %s\n", stringDetections.c_str());
                    // Дополнительно записываем в файл строк с контекстом CALL
                    // writeToStringsFile("Found at CALL instruction at " + hexToString(bp->breakpoint->addr), "AUTO-RUN:CALL");
                }

                // Дополнительная проверка аргументов CALL на строки
                for (size_t i = 0; i < args.size(); i++)
                {
                    if (DbgMemIsValidReadPtr(args[i]))
                    {
                        std::string strData = analyzeStringData(args[i]);
                        if (!strData.empty() && (strData[0] == '"' || strData.substr(0, 2) == "L\""))
                        {
                            std::string logMsg = "String found in argument " + std::to_string(i) + ": " + strData;
                            _plugin_logprintf("[" PLUGIN_NAME "] %s\n", logMsg.c_str());
                            GuiAddLogMessage(("[FunCap] " + logMsg + "\n").c_str());

                            // Получаем имя вызываемой функции
                            char funcName[MAX_LABEL_SIZE] = {0};
                            duint callTarget = 0;
                            BASIC_INSTRUCTION_INFO basicInfo = {0};
                            DbgDisasmFastAt(bp->breakpoint->addr, &basicInfo);
                            if (DbgGetLabelAt(DbgGetBranchDestination(bp->breakpoint->addr), SEG_DEFAULT, funcName))
                            {
                                writeToStringsFile(strData[0] == '"' ? strData.substr(1, strData.length() - 2) : strData.substr(2, strData.length() - 3),
                                                   "CALL:" + std::string(funcName) + ":ARG" + std::to_string(i));
                            }
                            else
                            {
                                writeToStringsFile(strData[0] == '"' ? strData.substr(1, strData.length() - 2) : strData.substr(2, strData.length() - 3),
                                                   "CALL:UNKNOWN:ARG" + std::to_string(i));
                            }
                        }
                    }
                }
            }

            // Предполагаем, что это вызов функции
            handleFunctionCall(bp->breakpoint->addr);

            // Удаляем точку останова, если нужно
            if (deleteBreakpoints)
            {
                _plugin_logprintf("[" PLUGIN_NAME "] Deleting CALL breakpoint at: 0x%p\n", (void *)bp->breakpoint->addr);

                // Skip system entry point breakpoints, they're managed by x64dbg
                if (bp->breakpoint->type == bp_normal && strstr(bp->breakpoint->mod, "entry") != NULL)
                {
                    _plugin_logprintf("[" PLUGIN_NAME "] Skipping deletion of entry point breakpoint\n");
                }
                else
                {
                    // Try different approaches for deleting breakpoints
                    char bpCmd[64] = {0};
                    sprintf_s(bpCmd, "bc %p", (void *)bp->breakpoint->addr);
                    _plugin_logprintf("[" PLUGIN_NAME "] Trying command: %s\n", bpCmd);
                    DbgCmdExec(bpCmd);

                    // For hardware breakpoints, try using bphc with the address (not handle)
                    if (bp->breakpoint->type != bp_normal)
                    {
                        sprintf_s(bpCmd, "bphc %p", (void *)bp->breakpoint->addr);
                        _plugin_logprintf("[" PLUGIN_NAME "] Trying hardware breakpoint delete command: %s\n", bpCmd);
                        DbgCmdExec(bpCmd);
                    }
                }
            }
        }

        // Если мы в режиме автозапуска, автоматически продолжаем выполнение
        if (autoRunMode)
        {
            // Принудительно сбрасываем буферы файлов перед продолжением
            if (logFile.is_open())
            {
                logFile.flush();
            }
            if (stringsFile.is_open())
            {
                stringsFile.flush();
            }

            // Небольшая задержка чтобы успеть обработать текущую точку останова
            Sleep(10);
            _plugin_logprintf("[" PLUGIN_NAME "] Auto-continuing execution in auto-run mode\n");
            DbgCmdExec("run");
        }
    }
    catch (const std::exception &e)
    {
        _plugin_logprintf("[" PLUGIN_NAME "] Exception in breakpoint callback: %s\n", e.what());
    }
    catch (...)
    {
        _plugin_logprintf("[" PLUGIN_NAME "] Unknown exception in breakpoint callback\n");
    }

    return true;
}

// Обработчик системной точки останова
bool cbSystemBreakpoint(CBTYPE cbType, PLUG_CB_SYSTEMBREAKPOINT *info)
{
    // Это точка входа в программу, можно автоматически начать захват здесь
    return false;
}

// Обработчик событий отладчика
bool cbDebugEvent(CBTYPE cbType, PLUG_CB_DEBUGEVENT *info)
{
    if (!pluginRunning)
        return false;

    // Проверяем, завершен ли процесс
    if (info->DebugEvent->dwDebugEventCode == EXIT_PROCESS_DEBUG_EVENT)
    {
        _plugin_logprintf("[" PLUGIN_NAME "] Process terminated\n");

        // Если мы в режиме автозапуска, завершаем его
        if (autoRunMode)
        {
            autoRunMode = false;
            deleteBreakpoints = true; // Убедимся, что все BP будут удалены

            _plugin_logprintf("[" PLUGIN_NAME "] Process exited. Stopping automated run...\n");
            GuiAddLogMessage("[FunCap] Automated run completed. Process terminated.\n");

            // Закрываем лог
            logFile << "=== Automated Run Completed at " << getCurrentTime() << " ===" << std::endl;
            logFile << "Process terminated with exit code: " << info->DebugEvent->u.ExitProcess.dwExitCode << std::endl;
            logFile.flush();

            // Явно закрываем файл строк и сообщаем об этом
            if (stringsFile.is_open())
            {
                try
                {
                    stringsFile << "=== Automated Run Completed at " << getCurrentTime() << " ===" << std::endl;
                    stringsFile << "Process terminated with exit code: " << info->DebugEvent->u.ExitProcess.dwExitCode << std::endl;
                    stringsFile.flush();
                    stringsFile.close();
                    _plugin_logprintf("[" PLUGIN_NAME "] Strings file explicitly closed at end of auto-run\n");
                }
                catch (const std::exception &e)
                {
                    _plugin_logprintf("[" PLUGIN_NAME "] Error closing strings file: %s\n", e.what());
                }
            }
            else
            {
                _plugin_logprintf("[" PLUGIN_NAME "] Strings file was not open at end of auto-run\n");
            }

            _plugin_logprintf("[" PLUGIN_NAME "] Automated run completed. Process exited with code %d\n",
                              info->DebugEvent->u.ExitProcess.dwExitCode);
            GuiAddLogMessage("[FunCap] Auto-Run completed. Check log file for details.\n");

            // Если черный список был изменен в процессе автозапуска, восстанавливаем его из файла
            _plugin_logprintf("[" PLUGIN_NAME "] Refreshing module blacklist from file\n");
            loadBlacklistFromFile();
        }
    }

    return true;
}

// Обработчик меню
bool cbMenuEntry(CBTYPE cbType, PLUG_CB_MENUENTRY *info)
{
    switch (info->hEntry)
    {
    case 0: // Start Capture
        if (!pluginRunning)
        {
            // Если файл уже открыт, закроем его
            if (logFile.is_open())
            {
                logFile.close();
            }
            if (stringsFile.is_open())
            {
                stringsFile.close();
            }

            // Сначала проверим доступность файла для записи
            bool fileAccessible = false;
            std::ofstream testFile(logFilePath, std::ios::out | std::ios::app);
            if (testFile.is_open())
            {
                testFile.close();
                fileAccessible = true;
            }

            if (!fileAccessible)
            {
                // Если не получается открыть файл, попробуем альтернативное расположение
                char tempPath[MAX_PATH] = {0};
                GetTempPathA(MAX_PATH, tempPath);
                std::string newPath = std::string(tempPath) + DEFAULT_LOG_FILENAME;
                std::string newStringsPath = std::string(tempPath) + DEFAULT_STRINGS_FILENAME;

                _plugin_logprintf("[" PLUGIN_NAME "] Cannot access original log file. Trying temp path: %s\n", newPath.c_str());

                std::ofstream tempTest(newPath, std::ios::out | std::ios::app);
                if (tempTest.is_open())
                {
                    tempTest.close();
                    logFilePath = newPath;
                    stringsFilePath = newStringsPath;
                    fileAccessible = true;
                }
                else
                {
                    _plugin_logprintf("[" PLUGIN_NAME "] WARNING: Cannot create log file in any location!\n");
                }
            }

            // Теперь открываем файл для реального использования
            logFile.open(logFilePath, std::ios::out | std::ios::trunc);
            // Открываем файл строк в режиме добавления, чтобы дописывать в него, а не перезаписывать
            stringsFile.open(stringsFilePath, std::ios::out | std::ios::app);

            if (logFile.is_open())
            {
                pluginRunning = true;

                try
                {
                    std::string startMessage = "=== FunCap Capture Started at " + getCurrentTime() + " ===";
                    logFile << startMessage << std::endl;
                    logFile.flush();

                    if (stringsFile.is_open())
                    {
                        stringsFile << "=== FunCap Strings Capture Started at " << getCurrentTime() << " ===" << std::endl;
                        stringsFile.flush();
                        _plugin_logprintf("[" PLUGIN_NAME "] Strings file: %s\n", stringsFilePath.c_str());
                    }
                    else
                    {
                        _plugin_logprintf("[" PLUGIN_NAME "] WARNING: Could not open strings file: %s\n", stringsFilePath.c_str());
                    }

                    _plugin_logprintf("[" PLUGIN_NAME "] Capture started. Log file: %s\n", logFilePath.c_str());
                    _plugin_logprintf("[" PLUGIN_NAME "] First log entry written successfully!\n");

                    // Выводим расширенную отладочную информацию
                    logFile << "Plugin version: " << PLUGIN_VERSION << std::endl;
                    logFile << "Architecture: " << (isX64 ? "x64" : "x86") << std::endl;
                    logFile << "Log path: " << logFilePath << std::endl;
                    logFile << "Strings path: " << stringsFilePath << std::endl;
                    logFile << std::endl;
                    logFile.flush();
                }
                catch (const std::exception &e)
                {
                    _plugin_logprintf("[" PLUGIN_NAME "] ERROR: Exception while writing initial log entries: %s\n", e.what());
                }

                // После успешного запуска добавляем текущие точки останова
                _plugin_logprintf("[" PLUGIN_NAME "] Adding existing breakpoints to log...\n");

                // Поскольку нет прямого API для перебора точек останова, мы можем:
                // 1. Записать информацию о запуске
                addLogLine("=== FunCap Plugin Started - Breakpoints will be logged when hit ===");
                _plugin_logprintf("[" PLUGIN_NAME "] Plugin started and ready to log breakpoints when hit\n");
            }
            else
            {
                _plugin_logprintf("[" PLUGIN_NAME "] Failed to open log file: %s\n", logFilePath.c_str());
                _plugin_logprintf("[" PLUGIN_NAME "] Make sure you have write permissions to this location\n");

                // Отображаем сообщение пользователю
                GuiAddLogMessage("[FunCap] ERROR: Cannot create log file. Please check permissions or specify a different path.\n");
            }
        }
        else
        {
            // Если плагин уже запущен, выводим сообщение
            _plugin_logprintf("[" PLUGIN_NAME "] Plugin is already running\n");
            GuiAddLogMessage("[FunCap] Plugin is already running. Stop it first to restart capture.\n");
        }
        break;
    case 1: // Stop Capture
        if (pluginRunning)
        {
            addLogLine("=== FunCap Capture Stopped at " + getCurrentTime() + " ===");
            if (logFile.is_open())
            {
                logFile.close();
            }
            if (stringsFile.is_open())
            {
                stringsFile << "=== FunCap Strings Capture Stopped at " << getCurrentTime() << " ===" << std::endl;
                stringsFile.close();
            }
            pluginRunning = false;
            _plugin_logprintf("[" PLUGIN_NAME "] Capture stopped\n");
        }
        break;
    case 2: // Add Breakpoints on All Functions
        menuAddFuncStart();
        break;
    case 3: // Add Breakpoints on All RET Instructions
        menuAddFuncRet();
        break;
    case 4: // Add Breakpoints on All CALL Instructions
        menuAddCallInstructions();
        break;
    case 5: // Delete All Breakpoints
        menuDeleteAllBreakpoints();
        break;
    case MENU_AUTO_RUN:
        menuAutoRunToEnd();
        break;
    case MENU_PAUSE_AUTO_RUN:
        menuPauseAutoRun();
        break;
    case MENU_MANAGE_BLACKLIST:
        menuManageBlacklist();
        break;
    case MENU_TEST_READ_STRINGS:
        menuTestReadStrings();
        break;
    }
    return false;
}

// Добавить точки останова на все функции
void menuAddFuncStart()
{
    _plugin_logprintf("[" PLUGIN_NAME "] Adding breakpoints on all function starts...\n");

    // Получаем список всех модулей
    std::vector<duint> moduleBases;
    BridgeList<Script::Module::ModuleInfo> moduleList;
    if (Script::Module::GetList(&moduleList))
    {
        for (int i = 0; i < moduleList.Count(); i++)
        {
            // Проверяем, не находится ли модуль в черном списке
            if (!isModuleBlacklisted(moduleList[i].name))
            {
                moduleBases.push_back(moduleList[i].base);
                _plugin_logprintf("[" PLUGIN_NAME "] Found module %s at base: 0x%p\n",
                                  moduleList[i].name, (void *)moduleList[i].base);
            }
            else
            {
                _plugin_logprintf("[" PLUGIN_NAME "] Skipping blacklisted module %s\n", moduleList[i].name);
            }
        }
    }
    else
    {
        _plugin_logprintf("[" PLUGIN_NAME "] Failed to get module list\n");
    }

    // Если не нашли модули, используем текущий модуль
    if (moduleBases.empty())
    {
        duint mainModuleBase = DbgFunctions()->ModBaseFromAddr(DbgValFromString("cip"));
        if (mainModuleBase)
        {
            moduleBases.push_back(mainModuleBase);
            char moduleName[MAX_MODULE_SIZE] = "";
            if (DbgFunctions()->ModNameFromAddr(mainModuleBase, moduleName, true))
            {
                _plugin_logprintf("[" PLUGIN_NAME "] Using current module: %s at base: 0x%p\n",
                                  moduleName, (void *)mainModuleBase);
            }
        }
    }

    // Принудительно запускаем анализ модулей, если функции не найдены
    _plugin_logprintf("[" PLUGIN_NAME "] Ensuring modules are analyzed for functions...\n");
    DbgCmdExec("analadv");
    Sleep(500); // Даем время для анализа

    // Перебираем модули и ищем функции
    int totalCount = 0;
    for (duint moduleBase : moduleBases)
    {
        char moduleName[MAX_MODULE_SIZE] = "";
        DbgFunctions()->ModNameFromAddr(moduleBase, moduleName, true);

        // Альтернативный подход - используем команду "funclist" и парсим результат
        char cmdBuf[256] = {0};
        sprintf_s(cmdBuf, "funcinfo %s", moduleName);

        // Команда вывода списка функций
        _plugin_logprintf("[" PLUGIN_NAME "] Retrieving function info for %s...\n", moduleName);

        // Пытаемся использовать Script API для получения функций
        int count = 0;
        BridgeList<Script::Function::FunctionInfo> functionList;
        if (Script::Function::GetList(&functionList) && functionList.Count() > 0)
        {
            for (int i = 0; i < functionList.Count(); i++)
            {
                // Проверяем, принадлежит ли функция текущему модулю
                if (strcmp(functionList[i].mod, moduleName) == 0)
                {
                    duint functionStart = functionList[i].rvaStart + moduleBase;
                    char cmd[256] = {0};
                    sprintf_s(cmd, "bp %p", (void *)functionStart);
                    DbgCmdExec(cmd);
                    count++;

                    // Добавляем задержку между установкой BP, чтобы не перегружать отладчик
                    if (count % 100 == 0)
                    {
                        _plugin_logprintf("[" PLUGIN_NAME "] Added %d breakpoints on %s...\n", count, moduleName);
                        Sleep(10); // Небольшая задержка
                    }
                }
            }
        }
        else
        {
            // Если не нашли функции через API, устанавливаем точки останова на ключевые адреса
            _plugin_logprintf("[" PLUGIN_NAME "] No functions found in %s via API, trying alternative method...\n", moduleName);

            // Устанавливаем точку останова на вход в модуль
            char cmd[256] = {0};
            sprintf_s(cmd, "bp %p", (void *)moduleBase);
            DbgCmdExec(cmd);
            count++;

            // Ищем экспортируемые функции в модуле
            char exportCmd[256] = {0};
            sprintf_s(exportCmd, "exportsanalysis %s", moduleName); // FIX: Added space between command and module name
            DbgCmdExec(exportCmd);

            // Ищем метку entry
            char entryLabel[MAX_LABEL_SIZE] = "";
            char entryCmd[256] = {0};
            if (DbgGetLabelAt(moduleBase, SEG_DEFAULT, entryLabel))
            {
                sprintf_s(entryCmd, "bp %s", entryLabel);
                DbgCmdExec(entryCmd);
                count++;
                _plugin_logprintf("[" PLUGIN_NAME "] Added breakpoint at entry label: %s\n", entryLabel);
            }

            // Если это основной модуль, добавляем точку останова на main/WinMain
            if (moduleBase == DbgFunctions()->ModBaseFromAddr(DbgValFromString("cip")))
            {
                DbgCmdExec("bp main");
                DbgCmdExec("bp WinMain");
                DbgCmdExec("bp _main");
                DbgCmdExec("bp entry");
                count += 4;
                _plugin_logprintf("[" PLUGIN_NAME "] Added breakpoints at common entry points\n");
            }
        }

        _plugin_logprintf("[" PLUGIN_NAME "] Added %d breakpoints on functions in module %s\n", count, moduleName);
        totalCount += count;
    }

    _plugin_logprintf("[" PLUGIN_NAME "] Added total of %d breakpoints on function starts\n", totalCount);
    _plugin_logprintf("[" PLUGIN_NAME "] Done\n");
}

// Добавить точки останова на все инструкции RET
void menuAddFuncRet()
{
    _plugin_logprintf("[" PLUGIN_NAME "] Adding breakpoints on all RET instructions...\n");

    // Получаем информацию о текущем исполняемом модуле
    duint cip = DbgValFromString("cip");
    duint moduleBase = DbgFunctions()->ModBaseFromAddr(cip);
    duint moduleSize = DbgFunctions()->ModSizeFromAddr(cip);

    if (moduleBase && moduleSize)
    {
        char moduleName[MAX_MODULE_SIZE] = "";
        DbgFunctions()->ModNameFromAddr(moduleBase, moduleName, true);

        // Проверяем, не в черном ли списке модуль
        if (isModuleBlacklisted(moduleName))
        {
            _plugin_logprintf("[" PLUGIN_NAME "] Module %s is blacklisted, skipping RET breakpoints\n", moduleName);
            GuiAddLogMessage(("[FunCap] Skipping RET breakpoints for blacklisted module " + std::string(moduleName) + "\n").c_str());
            return;
        }

        // Явная проверка на ntdll модуль
        std::string moduleNameLower = moduleName;
        std::transform(moduleNameLower.begin(), moduleNameLower.end(), moduleNameLower.begin(), ::tolower);
        if (moduleNameLower == "ntdll.dll")
        {
            _plugin_logprintf("[" PLUGIN_NAME "] Module ntdll.dll is explicitly blacklisted, skipping RET breakpoints\n");
            GuiAddLogMessage("[FunCap] Skipping RET breakpoints for ntdll.dll module\n");
            return;
        }

        duint end = moduleBase + moduleSize;
        _plugin_logprintf("[" PLUGIN_NAME "] Searching for RET instructions in module %s (range 0x%p - 0x%p)\n",
                          moduleName, (void *)moduleBase, (void *)end);

        // Сначала убедимся, что модуль загружен и проанализирован
        _plugin_logprintf("[" PLUGIN_NAME "] Ensuring module is analyzed...\n");
        char anaCmd[256] = {0};
        sprintf_s(anaCmd, "analadv %p", (void *)moduleBase);
        DbgCmdExec(anaCmd);
        Sleep(500); // Даем время для анализа

        // Более надежный способ поиска инструкций - используем внутренние команды x64dbg
        char findCmd[256] = {0};
        sprintf_s(findCmd, "findallmem %p, %p, c3|c2??|c2????", // RET, RETN xx, RETN xxxx
                  (void *)moduleBase, (void *)end);

        _plugin_logprintf("[" PLUGIN_NAME "] Executing search for ret patterns...\n");
        DbgCmdExec(findCmd);
        Sleep(500); // Даем время для поиска

        // Ищем возвраты через анализ дизассемблера
        int step = 1;
        duint addr = moduleBase;
        int count = 0;

        // Ограничиваем поиск первыми 64K для скорости
        while (addr < end && addr < moduleBase + 0x10000)
        {
            BASIC_INSTRUCTION_INFO basicInfo;
            if (dbgFunctions.DisasmFast(nullptr, addr, &basicInfo))
            {
                if (basicInfo.type == TYPE_RET)
                {
                    // Проверяем, не находится ли функция, которой принадлежит RET, в черном списке
                    bool isInFunction = false;
                    duint funcStart = 0;
                    duint funcEnd = 0;

                    if (Script::Function::Get(addr, &funcStart, &funcEnd))
                    {
                        isInFunction = true;

                        // Если начало функции находится в другом модуле, проверяем, не в черном ли списке этот модуль
                        if (funcStart != 0)
                        {
                            char funcModuleName[MAX_MODULE_SIZE] = "";
                            if (DbgFunctions()->ModNameFromAddr(funcStart, funcModuleName, true))
                            {
                                if (isModuleBlacklisted(funcModuleName))
                                {
                                    _plugin_logprintf("[" PLUGIN_NAME "] Skipping RET in blacklisted module function at 0x%p\n", (void *)addr);
                                    addr += basicInfo.size > 0 ? basicInfo.size : 1;
                                    continue; // Пропускаем RET в функции из черного списка
                                }
                            }
                        }
                    }

                    char cmd[256] = {0};
                    sprintf_s(cmd, "bp %p", (void *)addr);
                    DbgCmdExec(cmd);
                    count++;

                    if (count % 20 == 0)
                    {
                        _plugin_logprintf("[" PLUGIN_NAME "] Found %d RET instructions...\n", count);
                        Sleep(10);
                    }
                }

                addr += basicInfo.size > 0 ? basicInfo.size : 1;
            }
            else
            {
                addr += step;
            }
        }

        // Также находим и устанавливаем точки останова на возвраты из основных функций
        BridgeList<Script::Function::FunctionInfo> functionList;
        if (Script::Function::GetList(&functionList) && functionList.Count() > 0)
        {
            _plugin_logprintf("[" PLUGIN_NAME "] Looking for function returns in %d functions...\n", functionList.Count());

            for (int i = 0; i < functionList.Count(); i++)
            {
                // Проверяем, принадлежит ли функция текущему модулю и не в черном ли списке модуль
                if (strcmp(functionList[i].mod, moduleName) == 0)
                {
                    duint funcStart = functionList[i].rvaStart + moduleBase;
                    duint funcEnd = functionList[i].rvaEnd + moduleBase;

                    // Найти последнюю инструкцию в функции, которая обычно RET
                    duint lastAddr = funcEnd - 1;
                    while (lastAddr > funcStart)
                    {
                        BASIC_INSTRUCTION_INFO basicInfo;
                        if (dbgFunctions.DisasmFast(nullptr, lastAddr, &basicInfo) && basicInfo.type == TYPE_RET)
                        {
                            char cmd[256] = {0};
                            sprintf_s(cmd, "bp %p", (void *)lastAddr);
                            DbgCmdExec(cmd);
                            count++;
                            break;
                        }
                        lastAddr--;
                    }
                }
            }
        }

        _plugin_logprintf("[" PLUGIN_NAME "] Added %d breakpoints on RET instructions\n", count);
    }
    else
    {
        _plugin_logprintf("[" PLUGIN_NAME "] Failed to get information about current module\n");
    }

    _plugin_logprintf("[" PLUGIN_NAME "] Done\n");
}

// Добавить точки останова на все инструкции CALL
void menuAddCallInstructions()
{
    _plugin_logprintf("[" PLUGIN_NAME "] Adding breakpoints on all CALL instructions...\n");

    // Получаем информацию о текущем исполняемом модуле
    duint cip = DbgValFromString("cip");
    duint moduleBase = DbgFunctions()->ModBaseFromAddr(cip);
    duint moduleSize = DbgFunctions()->ModSizeFromAddr(cip);

    if (moduleBase && moduleSize)
    {
        char moduleName[MAX_MODULE_SIZE] = "";
        DbgFunctions()->ModNameFromAddr(moduleBase, moduleName, true);

        // Проверяем, не в черном ли списке модуль
        if (isModuleBlacklisted(moduleName))
        {
            _plugin_logprintf("[" PLUGIN_NAME "] Module %s is blacklisted, skipping CALL breakpoints\n", moduleName);
            GuiAddLogMessage(("[FunCap] Skipping CALL breakpoints for blacklisted module " + std::string(moduleName) + "\n").c_str());
            return;
        }

        // Явная проверка на ntdll модуль
        std::string moduleNameLower = moduleName;
        std::transform(moduleNameLower.begin(), moduleNameLower.end(), moduleNameLower.begin(), ::tolower);
        if (moduleNameLower == "ntdll.dll")
        {
            _plugin_logprintf("[" PLUGIN_NAME "] Module ntdll.dll is explicitly blacklisted, skipping CALL breakpoints\n");
            GuiAddLogMessage("[FunCap] Skipping CALL breakpoints for ntdll.dll module\n");
            return;
        }

        duint end = moduleBase + moduleSize;
        _plugin_logprintf("[" PLUGIN_NAME "] Searching for CALL instructions in module %s (range 0x%p - 0x%p)\n",
                          moduleName, (void *)moduleBase, (void *)end);

        // Сначала убедимся, что модуль загружен и проанализирован
        _plugin_logprintf("[" PLUGIN_NAME "] Ensuring module is analyzed...\n");
        char anaCmd[256] = {0};
        sprintf_s(anaCmd, "analadv %p", (void *)moduleBase);
        DbgCmdExec(anaCmd);
        Sleep(500); // Даем время для анализа

        // Более надежный способ поиска инструкций - используем внутренние команды x64dbg
        char findCmd[256] = {0};
        sprintf_s(findCmd, "findallmem %p, %p, e8????????|e9????????|ff15????????|ff25????????",
                  (void *)moduleBase, (void *)end);

        _plugin_logprintf("[" PLUGIN_NAME "] Executing search for call patterns...\n");
        DbgCmdExec(findCmd);
        Sleep(500); // Даем время для поиска

        // Для простого тестирования добавим точки останова на известные функции
        int count = 0;

        // Добавим точки останова на некоторые стандартные API вызовы, если они не в черном списке
        const char *commonApiCalls[] = {
            "kernel32.CreateFileW", "kernel32.ReadFile", "kernel32.WriteFile",
            "kernel32.CreateProcessW", "kernel32.VirtualAlloc", "kernel32.VirtualFree",
            "kernel32.HeapAlloc", "kernel32.HeapFree", "kernel32.LoadLibraryW",
            "kernel32.GetProcAddress", "user32.MessageBoxW", "user32.CreateWindowExW",
            "msvcrt.malloc", "msvcrt.free", "msvcrt.printf", "msvcrt.scanf"};

        for (const char *api : commonApiCalls)
        {
            // Извлекаем имя модуля из строки API (до точки)
            std::string apiStr = api;
            size_t dotPos = apiStr.find('.');
            if (dotPos != std::string::npos)
            {
                std::string apiModule = apiStr.substr(0, dotPos) + ".dll";
                std::transform(apiModule.begin(), apiModule.end(), apiModule.begin(), ::tolower);

                // Проверяем, не находится ли модуль API в черном списке
                if (blacklistedModules.find(apiModule) != blacklistedModules.end())
                {
                    _plugin_logprintf("[" PLUGIN_NAME "] Skipping API in blacklisted module: %s\n", api);
                    continue; // Пропускаем API из черного списка
                }
            }

            char bpCmd[256] = {0};
            sprintf_s(bpCmd, "bp %s", api);
            DbgCmdExec(bpCmd);
            count++;
        }

        // Ищем вызовы через анализ дизассемблера с фиксированным шагом
        // (это не идеально, но может помочь найти некоторые вызовы)
        int step = 1; // Маленький шаг для более точного поиска, но медленнее
        duint addr = moduleBase;
        int foundCount = 0;

        while (addr < end && addr < moduleBase + 0x10000) // Ограничиваем поиск первыми 64K для скорости
        {
            BASIC_INSTRUCTION_INFO basicInfo;
            if (dbgFunctions.DisasmFast(nullptr, addr, &basicInfo))
            {
                if (basicInfo.type == TYPE_CALL)
                {
                    // Проверяем, не указывает ли вызов на модуль из черного списка
                    bool isCallToBlacklisted = false;

                    if (basicInfo.addr != 0) // Если у нас есть целевой адрес вызова
                    {
                        char targetModuleName[MAX_MODULE_SIZE] = "";
                        if (DbgFunctions()->ModNameFromAddr(basicInfo.addr, targetModuleName, true))
                        {
                            if (isModuleBlacklisted(targetModuleName))
                            {
                                isCallToBlacklisted = true;
                            }
                        }
                    }

                    if (!isCallToBlacklisted)
                    {
                        char cmd[256] = {0};
                        sprintf_s(cmd, "bp %p", (void *)addr);
                        DbgCmdExec(cmd);
                        foundCount++;

                        if (foundCount % 20 == 0)
                        {
                            _plugin_logprintf("[" PLUGIN_NAME "] Found %d CALL instructions...\n", foundCount);
                            Sleep(10);
                        }
                    }
                }

                addr += basicInfo.size > 0 ? basicInfo.size : 1;
            }
            else
            {
                addr += step;
            }
        }

        count += foundCount;
        _plugin_logprintf("[" PLUGIN_NAME "] Added %d breakpoints on CALL instructions and APIs\n", count);
    }
    else
    {
        _plugin_logprintf("[" PLUGIN_NAME "] Failed to get information about current module\n");
    }

    _plugin_logprintf("[" PLUGIN_NAME "] Done\n");
}

// Удалить все точки останова
void menuDeleteAllBreakpoints()
{
    _plugin_logprintf("[" PLUGIN_NAME "] Deleting all breakpoints...\n");
    DbgCmdExec("bc");
    _plugin_logprintf("[" PLUGIN_NAME "] Done\n");
}

// Переключить логирование
void menuToggleLogging()
{
    if (logFile.is_open())
    {
        try
        {
            addLogLine("=== FunCap Logging Disabled at " + getCurrentTime() + " ===");
            logFile.close();
            _plugin_logprintf("[" PLUGIN_NAME "] Logging disabled\n");
        }
        catch (const std::exception &e)
        {
            _plugin_logprintf("[" PLUGIN_NAME "] Error closing log file: %s\n", e.what());
        }
    }
    else
    {
        try
        {
            logFile.open(logFilePath, std::ios::out | std::ios::app);
            if (logFile.is_open())
            {
                std::string message = "=== FunCap Logging Enabled at " + getCurrentTime() + " ===";
                logFile << message << std::endl;
                logFile.flush();

                _plugin_logprintf("[" PLUGIN_NAME "] Logging enabled. Log file: %s\n", logFilePath.c_str());
                _plugin_logprintf("[" PLUGIN_NAME "] Successfully wrote to log file\n");
            }
            else
            {
                _plugin_logprintf("[" PLUGIN_NAME "] Failed to open log file: %s\n", logFilePath.c_str());

                // Пробуем создать файл в том же каталоге, что и плагин
                char currentDir[MAX_PATH] = {0};
                GetCurrentDirectoryA(MAX_PATH, currentDir);
                std::string newPath = std::string(currentDir) + "\\funcap_new.txt";

                _plugin_logprintf("[" PLUGIN_NAME "] Attempting to use alternative log file: %s\n", newPath.c_str());

                logFile.open(newPath, std::ios::out);
                if (logFile.is_open())
                {
                    logFilePath = newPath;
                    logFile << "=== FunCap Log File (Alternative) - Created at " << getCurrentTime() << " ===" << std::endl;
                    logFile.flush();
                    _plugin_logprintf("[" PLUGIN_NAME "] Successfully opened alternative log file\n");
                }
                else
                {
                    _plugin_logprintf("[" PLUGIN_NAME "] All attempts to create log file failed!\n");
                }
            }
        }
        catch (const std::exception &e)
        {
            _plugin_logprintf("[" PLUGIN_NAME "] Error opening log file: %s\n", e.what());
        }
    }
}

// Получить текущее время в формате строки
std::string getCurrentTime()
{
    std::time_t now = std::time(nullptr);
    char buffer[80];
    std::strftime(buffer, sizeof(buffer), "%Y-%m-%d %H:%M:%S", std::localtime(&now));
    return std::string(buffer);
}

// Добавить строку в лог-файл
void addLogLine(const std::string &line)
{
    if (!pluginRunning)
    {
        // Плагин не запущен, не нужно логировать
        return;
    }

    if (!logFile.is_open())
    {
        _plugin_logprintf("[" PLUGIN_NAME "] Warning: Attempted to log but file is not open. Opening now...\n");
        try
        {
            logFile.open(logFilePath, std::ios::out | std::ios::app);
            if (!logFile.is_open())
            {
                _plugin_logprintf("[" PLUGIN_NAME "] ERROR: Failed to open log file: %s\n", logFilePath.c_str());
                return;
            }
            logFile << "=== Log file reopened at " << getCurrentTime() << " ===" << std::endl;
        }
        catch (const std::exception &e)
        {
            _plugin_logprintf("[" PLUGIN_NAME "] Exception opening log file: %s\n", e.what());
            return;
        }
    }

    try
    {
        // Output to x64dbg log as well for debugging purposes
        _plugin_logprintf("[" PLUGIN_NAME "] Writing to log: %s\n",
                          line.substr(0, 30).c_str()); // Show first 30 chars for debug

        logFile << line << std::endl;
        // Explicitly flush the log file after each write
        logFile.flush();

        // If you detect a string, log it separately with a highlight
        if (line.find("\"") != std::string::npos)
        {
            _plugin_logprintf("[" PLUGIN_NAME "] String detected in log entry\n");
        }

        // If we got this far, writing to log was successful
        _plugin_logprintf("[" PLUGIN_NAME "] Successfully wrote %d bytes to log\n", (int)line.length());
    }
    catch (const std::exception &e)
    {
        _plugin_logprintf("[" PLUGIN_NAME "] ERROR writing to log: %s\n", e.what());
    }
}

// Проверить, является ли инструкция вызовом функции
bool isCallInstruction(duint addr)
{
    BASIC_INSTRUCTION_INFO basicInfo;
    if (dbgFunctions.DisasmFast(nullptr, addr, &basicInfo))
    {
        return basicInfo.type == TYPE_CALL;
    }
    return false;
}

// Проверить, является ли инструкция возвратом из функции
bool isRetInstruction(duint addr)
{
    BASIC_INSTRUCTION_INFO basicInfo;
    if (dbgFunctions.DisasmFast(nullptr, addr, &basicInfo))
    {
        return basicInfo.type == TYPE_RET;
    }
    return false;
}

// Получить адрес возврата (со стека)
duint getReturnAddress()
{
    duint rsp = 0;
    bool success = false;
    duint result = DbgEval("rsp", &success);
    if (!success)
        return 0;

    rsp = result;
    duint returnAddr = 0;
    if (isX64)
    {
        DbgMemRead(rsp, &returnAddr, 8);
    }
    else
    {
        DbgMemRead(rsp, &returnAddr, 4);
    }
    return returnAddr;
}

// Получить текущее состояние регистров
std::vector<RegisterContext> getRegisters()
{
    std::vector<RegisterContext> registers;

    // Получаем значения регистров через DbgEval
    bool success = false;

    if (isX64)
    {
        // x64 registers
        registers.push_back({"RAX", DbgEval("rax", &success)});
        registers.push_back({"RBX", DbgEval("rbx", &success)});
        registers.push_back({"RCX", DbgEval("rcx", &success)});
        registers.push_back({"RDX", DbgEval("rdx", &success)});
        registers.push_back({"RSI", DbgEval("rsi", &success)});
        registers.push_back({"RDI", DbgEval("rdi", &success)});
        registers.push_back({"RBP", DbgEval("rbp", &success)});
        registers.push_back({"RSP", DbgEval("rsp", &success)});
        registers.push_back({"RIP", DbgEval("rip", &success)});
        registers.push_back({"R8", DbgEval("r8", &success)});
        registers.push_back({"R9", DbgEval("r9", &success)});
        registers.push_back({"R10", DbgEval("r10", &success)});
        registers.push_back({"R11", DbgEval("r11", &success)});
        registers.push_back({"R12", DbgEval("r12", &success)});
        registers.push_back({"R13", DbgEval("r13", &success)});
        registers.push_back({"R14", DbgEval("r14", &success)});
        registers.push_back({"R15", DbgEval("r15", &success)});
    }
    else
    {
        // x86 registers
        registers.push_back({"EAX", DbgEval("eax", &success)});
        registers.push_back({"EBX", DbgEval("ebx", &success)});
        registers.push_back({"ECX", DbgEval("ecx", &success)});
        registers.push_back({"EDX", DbgEval("edx", &success)});
        registers.push_back({"ESI", DbgEval("esi", &success)});
        registers.push_back({"EDI", DbgEval("edi", &success)});
        registers.push_back({"EBP", DbgEval("ebp", &success)});
        registers.push_back({"ESP", DbgEval("esp", &success)});
        registers.push_back({"EIP", DbgEval("eip", &success)});
    }

    return registers;
}

// Получить аргументы функции со стека или из регистров
std::vector<duint> getArguments(duint functionAddr, int numArgs)
{
    std::vector<duint> args;
    bool success = false;

    if (isX64)
    {
        // В x64 первые 4 аргумента передаются через регистры
        if (numArgs >= 1)
            args.push_back(DbgEval("rcx", &success)); // RCX
        if (numArgs >= 2)
            args.push_back(DbgEval("rdx", &success)); // RDX
        if (numArgs >= 3)
            args.push_back(DbgEval("r8", &success)); // R8
        if (numArgs >= 4)
            args.push_back(DbgEval("r9", &success)); // R9

        // Остальные аргументы берутся со стека (RSP + 32 + i*8)
        duint rsp = DbgEval("rsp", &success);
        for (int i = 4; i < numArgs; i++)
        {
            duint arg = 0;
            if (DbgMemRead(rsp + 32 + (i - 4) * 8, &arg, 8))
            {
                args.push_back(arg);
            }
            else
            {
                args.push_back(0);
            }
        }
    }
    else
    {
        // В x86 аргументы передаются через стек (ESP + 4 + i*4)
        duint esp = DbgEval("esp", &success);

        for (int i = 0; i < numArgs; i++)
        {
            duint arg = 0;
            if (DbgMemRead(esp + 4 + i * 4, &arg, 4))
            {
                args.push_back(arg);
            }
            else
            {
                args.push_back(0);
            }
        }
    }

    return args;
}

// Получить ASCII строку из памяти
std::string getAsciiString(const std::vector<unsigned char> &data)
{
    std::string result;

    for (unsigned char c : data)
    {
        if (c < 32 || c > 126)
        {
            break;
        }
        result += static_cast<char>(c);
    }

    if (result.length() < STRING_EXPLORATION_MIN_LENGTH)
    {
        return "";
    }

    return result;
}

// Получить печатную строку из данных
std::string getPrintableString(const std::vector<unsigned char> &data, bool printDots)
{
    std::string result;

    for (unsigned char c : data)
    {
        if (c >= 32 && c <= 126)
        {
            result += static_cast<char>(c);
        }
        else if (printDots)
        {
            result += ".";
        }
    }

    return result;
}

// Чтение памяти по адресу
std::vector<unsigned char> readMemory(duint address, size_t size)
{
    std::vector<unsigned char> result(size, 0);
    if (!DbgMemRead(address, result.data(), size))
    {
        result.clear();
    }
    return result;
}

// Форматировать содержимое памяти
std::string smartFormat(duint address, int maxLen)
{
    if (address == 0)
    {
        return "NULL";
    }

    std::vector<unsigned char> data = readMemory(address, maxLen * 2);
    if (data.empty())
    {
        return "???";
    }

    // Попробуем интерпретировать как строку
    std::string asciiStr = getAsciiString(data);
    if (!asciiStr.empty())
    {
        if (asciiStr.length() > maxLen)
        {
            asciiStr = asciiStr.substr(0, maxLen - 3) + "...";
        }
        return asciiStr;
    }

    // Иначе вернем шестнадцатеричный дамп
    std::stringstream ss;
    int count = 0;

    for (unsigned char c : data)
    {
        if (count >= maxLen / 2)
        {
            break;
        }
        ss << std::hex << std::setw(2) << std::setfill('0') << static_cast<int>(c) << " ";
        count++;
    }

    return ss.str();
}

// Получить имя функции по адресу
bool getFunctionNameAtAddress(duint addr, char *name, int maxNameSize)
{
    char moduleName[MAX_MODULE_SIZE] = "";
    char symbolName[MAX_LABEL_SIZE] = "";
    int modbase = 0;

    bool modResult = dbgFunctions.ModNameFromAddr(addr, moduleName, true);
    modbase = (int)dbgFunctions.ModBaseFromAddr(addr);

    // Пытаемся получить символьное имя
    if (DbgGetLabelAt(addr, SEG_DEFAULT, symbolName))
    {
        if (modResult && strlen(moduleName) > 0)
        {
            snprintf(name, maxNameSize, "%s.%s", moduleName, symbolName);
        }
        else
        {
            strncpy_s(name, maxNameSize, symbolName, _TRUNCATE);
        }
        return true;
    }

    // Если не нашли имя, генерируем его по смещению в модуле
    if (modbase)
    {
        snprintf(name, maxNameSize, "%s+0x%llX", moduleName, (unsigned long long)(addr - modbase));
        return true;
    }

    // Если ничего не нашли, используем адрес
    snprintf(name, maxNameSize, "0x%p", reinterpret_cast<void *>(addr));
    return false;
}

// Найти начало функции по адресу внутри функции
duint findFunctionStart(duint addr)
{
    duint start = 0;
    duint end = 0;

    if (Script::Function::Get(addr, &start, &end))
    {
        return start;
    }

    return addr; // Если не удалось найти функцию, возвращаем исходный адрес
}

// Обработчик вызова функции
void handleFunctionCall(duint addr)
{
    if (!pluginRunning)
    {
        _plugin_logprintf("[" PLUGIN_NAME "] Warning: handleFunctionCall called while plugin is not running\n");
        return;
    }

    try
    {
        if (std::find(visitedAddresses.begin(), visitedAddresses.end(), addr) != visitedAddresses.end() && !deleteBreakpoints)
        {
            _plugin_logprintf("[" PLUGIN_NAME "] Address already visited: 0x%p\n", (void *)addr);
            return; // Уже посещали этот адрес, и не удаляем точки останова
        }

        visitedAddresses.push_back(addr);

        // Получаем информацию о вызываемой функции
        BASIC_INSTRUCTION_INFO basicInfo;
        if (!dbgFunctions.DisasmFast(nullptr, addr, &basicInfo))
        {
            _plugin_logprintf("[" PLUGIN_NAME "] Failed to disassemble instruction at 0x%p\n", (void *)addr);
            return;
        }

        duint targetAddr = basicInfo.addr;
        duint returnAddr = addr + basicInfo.size;

        // Получаем имя функции
        char functionName[MAX_PATH] = "";
        if (!getFunctionNameAtAddress(targetAddr, functionName, MAX_PATH))
        {
            // Не смогли получить имя, используем адрес
            sprintf_s(functionName, "func_0x%llX", (unsigned long long)targetAddr);
            _plugin_logprintf("[" PLUGIN_NAME "] Could not get function name at 0x%p, using address\n", (void *)targetAddr);
        }

        // Получаем регистры и аргументы
        std::vector<RegisterContext> registers;
        try
        {
            registers = getRegisters();
        }
        catch (const std::exception &e)
        {
            _plugin_logprintf("[" PLUGIN_NAME "] Error getting registers: %s\n", e.what());
        }

        std::vector<duint> args;
        try
        {
            args = getArguments(targetAddr, MAX_ARGS);
        }
        catch (const std::exception &e)
        {
            _plugin_logprintf("[" PLUGIN_NAME "] Error getting arguments: %s\n", e.what());
        }

        // Сохраняем информацию о вызове
        FunctionCall call;
        call.callerAddr = addr;
        call.functionAddr = targetAddr;
        call.returnAddr = returnAddr;
        call.functionName = functionName;
        call.arguments = args;
        call.processed = false;

        // Сохраняем состояние регистров
        for (const auto &reg : registers)
        {
            call.registers[reg.name] = reg.value;
        }

        // Добавляем точку останова на адрес возврата
        if (!deleteBreakpoints)
        {
            _plugin_logprintf("[" PLUGIN_NAME "] Setting breakpoint at return address: 0x%p\n", (void *)returnAddr);
            char cmd[64] = {0};
            sprintf_s(cmd, "bp %p", (void *)returnAddr);
            DbgCmdExec(cmd);
        }

        // Сохраняем информацию для последующей обработки возврата
        activeFunctionCalls[returnAddr] = call;

        // Подсвечиваем адрес вызова, если включено
        if (colorEnable)
        {
            DbgSetBookmarkAt(addr, true);
            DbgSetCommentAt(addr, ("Call to " + std::string(functionName)).c_str());
        }

        // Логируем информацию о вызове
        std::stringstream ss;
        ss << "=== Function Call at 0x" << std::hex << addr << " ===" << std::endl;
        ss << "Calling function " << functionName << " at 0x" << std::hex << targetAddr << std::endl;
        ss << "Return address: 0x" << std::hex << returnAddr << std::endl;

        // Выводим регистры
        ss << "Register state:" << std::endl;
        for (const auto &reg : registers)
        {
            ss << "  " << reg.name << " = 0x" << std::hex << reg.value;

            // Используем улучшенный анализ строк
            if (reg.value != 0)
            {
                std::string strData = analyzeStringData(reg.value);
                if (!strData.empty() && strData[0] == '"')
                {
                    ss << " -> " << strData;
                }
            }
            ss << std::endl;
        }

        // Выводим аргументы и их возможные интерпретации
        ss << "Arguments:" << std::endl;
        for (size_t i = 0; i < args.size(); i++)
        {
            duint arg = args[i];
            ss << "  arg" << i << " = 0x" << std::hex << arg;

            // Используем улучшенный анализ строк для аргументов
            if (arg != 0)
            {
                std::string strData = analyzeStringData(arg);
                if (!strData.empty())
                {
                    ss << " -> " << strData;
                }
            }
            ss << std::endl;
        }

        // Extract the string detection section from within the try block, before addLogLine(ss.str())
        std::string stringDetections = formatDetectedStrings(registers, args);
        if (!stringDetections.empty())
        {
            ss << std::endl
               << stringDetections << std::endl;

            // Add to GUI log for immediate feedback
            GuiAddLogMessage(("[FunCap] Strings detected in call to " + std::string(functionName) + "\n").c_str());

            // Явно записываем сводку обнаружения строк в лог-файл
            _plugin_logprintf("[" PLUGIN_NAME "] String detection info: %s\n", stringDetections.c_str());

            // Принудительно записываем информацию в файл строк для надежности
            writeToStringsFile("======= STRINGS DETECTED in call to " + std::string(functionName) + " at address 0x" +
                                   hexToString(addr).substr(2) + " =======",
                               "CALL");

            // Если в автоматическом режиме, флушим файл чаще
            if (autoRunMode)
            {
                if (stringsFile.is_open())
                {
                    stringsFile.flush();
                    _plugin_logprintf("[" PLUGIN_NAME "] Flushed strings file in auto-run mode\n");
                }
            }
        }

        // Добавляем в лог с более явным выводом на консоль
        _plugin_logprintf("[" PLUGIN_NAME "] Logging function call: %s with %zu args\n",
                          functionName, args.size());
        addLogLine(ss.str());

        // Дополнительный отладочный вывод
        GuiAddLogMessage(("[FunCap] Call to " + std::string(functionName) + "\n").c_str());
    }
    catch (const std::exception &e)
    {
        _plugin_logprintf("[" PLUGIN_NAME "] Exception in handleFunctionCall: %s\n", e.what());
    }
    catch (...)
    {
        _plugin_logprintf("[" PLUGIN_NAME "] Unknown exception in handleFunctionCall\n");
    }
}

// Обработчик возврата из функции
void handleFunctionReturn(duint addr)
{
    if (!pluginRunning)
    {
        _plugin_logprintf("[" PLUGIN_NAME "] Warning: handleFunctionReturn called while plugin is not running\n");
        return;
    }

    try
    {
        // Если это адрес возврата из вызова, который мы отслеживаем
        auto it = activeFunctionCalls.find(addr);
        if (it != activeFunctionCalls.end())
        {
            FunctionCall &call = it->second;

            if (call.processed)
            {
                _plugin_logprintf("[" PLUGIN_NAME "] Return at 0x%p already processed, skipping\n", (void *)addr);
                return; // Уже обработали этот возврат
            }

            // Получаем возвращаемое значение (обычно в RAX/EAX)
            bool success = false;
            duint returnValue = 0;

            try
            {
                returnValue = DbgEval(isX64 ? "rax" : "eax", &success);
                if (!success)
                {
                    _plugin_logprintf("[" PLUGIN_NAME "] Failed to evaluate return value at 0x%p\n", (void *)addr);
                }
            }
            catch (...)
            {
                _plugin_logprintf("[" PLUGIN_NAME "] Exception evaluating return value\n");
                success = false;
            }

            call.returnValue = returnValue;
            call.processed = true; // Помечаем как обработанный

            // Получаем текущее состояние регистров
            std::vector<RegisterContext> registers;
            try
            {
                registers = getRegisters();
            }
            catch (const std::exception &e)
            {
                _plugin_logprintf("[" PLUGIN_NAME "] Error getting registers in return handler: %s\n", e.what());
            }

            // Усиленное обнаружение строк - проверяем все регистры
            std::vector<duint> argsForStrings; // Пустой вектор аргументов, т.к. на возврате нас интересуют строки только из регистров

            // Явная проверка каждого регистра на наличие строк
            for (const auto &reg : registers)
            {
                if (reg.value != 0 && DbgMemIsValidReadPtr(reg.value))
                {
                    std::string strData = analyzeStringData(reg.value);
                    if (!strData.empty() && (strData[0] == '"' || strData.substr(0, 2) == "L\""))
                    {
                        // Получаем строку без кавычек для записи
                        std::string cleanStr = strData;
                        if (cleanStr[0] == '"')
                            cleanStr = cleanStr.substr(1, cleanStr.length() - 2);
                        else if (cleanStr.substr(0, 2) == "L\"")
                            cleanStr = cleanStr.substr(2, cleanStr.length() - 3);

                        _plugin_logprintf("[" PLUGIN_NAME "] String found in %s at return from %s: %s\n",
                                          reg.name.c_str(), call.functionName.c_str(), cleanStr.c_str());

                        // Записываем строку в файл с контекстом возврата
                        std::string context = "RETURN:" + call.functionName + ":" + reg.name;
                        writeToStringsFile(cleanStr, context);
                    }
                }
            }

            // Также используем обычную функцию для комплексного анализа
            std::string stringDetections = formatDetectedStrings(registers, argsForStrings);
            if (!stringDetections.empty())
            {
                _plugin_logprintf("[" PLUGIN_NAME "] Strings detected in return from %s\n", call.functionName.c_str());
                GuiAddLogMessage(("[FunCap] Strings detected in return from " + call.functionName + "\n").c_str());
                _plugin_logprintf("[" PLUGIN_NAME "] String detection info: %s\n", stringDetections.c_str());

                // Принудительно записываем информацию в файл строк для надежности
                writeToStringsFile("======= STRINGS DETECTED in return from " + call.functionName + " at address 0x" +
                                       hexToString(addr).substr(2) + " =======",
                                   "RETURN");

                // Если в автоматическом режиме, флушим файл чаще
                if (autoRunMode)
                {
                    if (stringsFile.is_open())
                    {
                        stringsFile.flush();
                        _plugin_logprintf("[" PLUGIN_NAME "] Flushed strings file in auto-run mode (return handler)\n");
                    }
                }
            }

            // Additional check for return value if it's a valid pointer
            if (success && returnValue != 0 && DbgMemIsValidReadPtr(returnValue))
            {
                std::string strData = analyzeStringData(returnValue);
                if (!strData.empty() && (strData[0] == '"' || strData.substr(0, 2) == "L\""))
                {
                    // Get clean string without quotes
                    std::string cleanStr = strData;
                    if (cleanStr[0] == '"')
                        cleanStr = cleanStr.substr(1, cleanStr.length() - 2);
                    else if (cleanStr.substr(0, 2) == "L\"")
                        cleanStr = cleanStr.substr(2, cleanStr.length() - 3);

                    _plugin_logprintf("[" PLUGIN_NAME "] String found in return value from %s: %s\n",
                                      call.functionName.c_str(), cleanStr.c_str());

                    // Write to string file with return value context
                    std::string context = "RETURN_VALUE:" + call.functionName;
                    writeToStringsFile(cleanStr, context);

                    // Add to GUI log for immediate feedback
                    GuiAddLogMessage(("[FunCap] String in return value from " + call.functionName + ": " + cleanStr + "\n").c_str());
                }
            }

            // Подсвечиваем, если включено
            if (colorEnable)
            {
                try
                {
                    DbgSetCommentAt(addr, ("Return from " + call.functionName).c_str());
                }
                catch (...)
                {
                    _plugin_logprintf("[" PLUGIN_NAME "] Error setting comment at return address\n");
                }
            }

            // Логируем информацию о возврате
            std::stringstream ss;
            ss << "=== Function Return at 0x" << std::hex << addr << " ===" << std::endl;
            ss << "Returned from " << call.functionName << " (called at 0x" << std::hex << call.callerAddr << ")" << std::endl;

            if (success)
            {
                ss << "Return value: 0x" << std::hex << returnValue;

                // Analyze return value with improved string detection
                if (returnValue != 0)
                {
                    std::string strData = analyzeStringData(returnValue);
                    if (!strData.empty())
                    {
                        ss << " -> " << strData;

                        // If it's a string, add it to GUI log
                        if (strData[0] == '"' || strData[0] == 'L')
                        {
                            GuiAddLogMessage(("[FunCap] String returned from " + call.functionName + ": " + strData + "\n").c_str());
                        }
                    }
                }
            }
            else
            {
                ss << "Return value: [ERROR READING VALUE]";
            }

            ss << std::endl;

            // Выводим измененные регистры
            ss << "Changed registers:" << std::endl;
            bool anyChanged = false;

            for (const auto &reg : registers)
            {
                auto oldIt = call.registers.find(reg.name);
                if (oldIt != call.registers.end() && oldIt->second != reg.value)
                {
                    ss << "  " << reg.name << ": 0x" << std::hex << oldIt->second << " -> 0x" << reg.value << std::endl;
                    anyChanged = true;
                }
            }

            if (!anyChanged)
            {
                ss << "  [No registers changed]" << std::endl;
            }

            ss << std::endl;

            try
            {
                addLogLine(ss.str());
                _plugin_logprintf("[" PLUGIN_NAME "] Successfully logged function return from %s\n", call.functionName.c_str());
            }
            catch (const std::exception &e)
            {
                _plugin_logprintf("[" PLUGIN_NAME "] Error writing function return to log: %s\n", e.what());
            }

            // Удаляем обработанный вызов из карты, если необходимо
            if (deleteBreakpoints)
            {
                activeFunctionCalls.erase(it);
            }
        }
        else
        {
            _plugin_logprintf("[" PLUGIN_NAME "] Return address 0x%p not found in tracked calls\n", (void *)addr);

            // Even if we don't have call information, still try to capture strings from registers
            std::vector<RegisterContext> registers = getRegisters();
            for (const auto &reg : registers)
            {
                if (reg.value != 0 && DbgMemIsValidReadPtr(reg.value))
                {
                    std::string strData = analyzeStringData(reg.value);
                    if (!strData.empty() && (strData[0] == '"' || strData.substr(0, 2) == "L\""))
                    {
                        // Get clean string without quotes
                        std::string cleanStr = strData;
                        if (cleanStr[0] == '"')
                            cleanStr = cleanStr.substr(1, cleanStr.length() - 2);
                        else if (cleanStr.substr(0, 2) == "L\"")
                            cleanStr = cleanStr.substr(2, cleanStr.length() - 3);

                        _plugin_logprintf("[" PLUGIN_NAME "] String found in %s at untracked return 0x%p: %s\n",
                                          reg.name.c_str(), (void *)addr, cleanStr.c_str());

                        // Write to string file with context
                        std::string context = "UNTRACKED_RETURN:" + reg.name;
                        writeToStringsFile(cleanStr, context);
                    }
                }
            }
        }
    }
    catch (const std::exception &e)
    {
        _plugin_logprintf("[" PLUGIN_NAME "] Exception in handleFunctionReturn: %s\n", e.what());
    }
    catch (...)
    {
        _plugin_logprintf("[" PLUGIN_NAME "] Unknown exception in handleFunctionReturn\n");
    }
}

// Improved function for smart analysis of potential string data
std::string analyzeStringData(duint address)
{
    if (address == 0)
        return "NULL";

    // Проверяем, является ли адрес действительным для чтения
    if (!DbgMemIsValidReadPtr(address))
        return "INVALID_PTR";

    // Сначала попробуем получить строку с помощью встроенной функции SDK
    char sdkString[STRING_LENGTH + 1] = {0};
    if (DbgGetStringAt(address, sdkString))
    {
        std::string result = sdkString;
        if (result.length() > STRING_LENGTH)
            result = result.substr(0, STRING_LENGTH - 3) + "...";
        return "\"" + result + "\"";
    }

    // Если SDK не смог получить строку, попробуем наш собственный метод

    // Сначала попробуем ASCII строку
    std::vector<unsigned char> data = readMemory(address, 1024); // Увеличиваем размер буфера
    if (data.empty())
        return "???";

    // Проверяем если это ASCII строка
    std::string asciiStr;
    bool validAscii = true;

    for (size_t i = 0; i < data.size(); i++)
    {
        if (data[i] == 0)
            break; // Найден нуль-терминатор

        if ((data[i] >= 32 && data[i] <= 126) || data[i] == '\r' || data[i] == '\n' || data[i] == '\t') // Печатаемые ASCII + некоторые управляющие символы
            asciiStr += static_cast<char>(data[i]);
        else
        {
            validAscii = false;
            break;
        }
    }

    if (validAscii && asciiStr.length() >= 3)
    {
        if (asciiStr.length() > STRING_LENGTH)
            asciiStr = asciiStr.substr(0, STRING_LENGTH - 3) + "...";
        return "\"" + asciiStr + "\"";
    }

    // Проверяем если это широкая строка (UTF-16LE)
    std::string wideStr;
    bool validWide = true;

    for (size_t i = 0; i < data.size(); i += 2)
    {
        if (i + 1 >= data.size())
            break;

        if (data[i] == 0 && data[i + 1] == 0)
            break; // Найден wide нуль-терминатор

        // Проверяем символы UTF-16LE (младший байт содержит символ ASCII, старший байт равен 0)
        if (data[i + 1] == 0 &&
            ((data[i] >= 32 && data[i] <= 126) || data[i] == '\r' || data[i] == '\n' || data[i] == '\t'))
            wideStr += static_cast<char>(data[i]);
        else
        {
            validWide = false;
            break;
        }
    }

    if (validWide && wideStr.length() >= 3)
    {
        if (wideStr.length() > STRING_LENGTH)
            wideStr = wideStr.substr(0, STRING_LENGTH - 3) + "...";
        return "L\"" + wideStr + "\"";
    }

    // Если строка не обнаружена, но адрес действителен,
    // выведем шестнадцатеричный дамп первых байт
    std::stringstream ss;
    int count = std::min<int>(16, static_cast<int>(data.size()));
    for (int i = 0; i < count; i++)
    {
        ss << std::hex << std::setw(2) << std::setfill('0') << static_cast<int>(data[i]) << " ";
    }
    return ss.str();
}

// Реализация функции автоматического запуска и логирования
void menuAutoRunToEnd()
{
    // Убедимся, что у нас есть точки останова и плагин включен
    if (!pluginRunning)
    {
        _plugin_logprintf("[" PLUGIN_NAME "] Starting plugin before auto-run...\n");

        // Открываем лог файл, если не открыт
        if (!logFile.is_open())
        {
            logFile.open(logFilePath, std::ios::out | std::ios::trunc);
            if (!logFile.is_open())
            {
                _plugin_logprintf("[" PLUGIN_NAME "] ERROR: Could not open log file for auto-run!\n");
                return;
            }
            logFile << "=== FunCap Capture Started at " << getCurrentTime() << " ===" << std::endl;
            logFile << "Plugin version: " << PLUGIN_VERSION << std::endl;
            logFile << "Architecture: " << (isX64 ? "x64" : "x86") << std::endl;
            logFile << "Log path: " << logFilePath << std::endl
                    << std::endl;

            // Записываем информацию о черном списке
            logFile << "Module blacklist: " << blacklistedModules.size() << " modules" << std::endl;
            for (const std::string &module : blacklistedModules)
            {
                logFile << "  - " << module << std::endl;
            }
            logFile << std::endl;

            logFile.flush();
        }

        // Принудительно закрываем и открываем заново файл строк для гарантированной работы
        if (stringsFile.is_open())
        {
            try
            {
                stringsFile.flush();
                stringsFile.close();
                _plugin_logprintf("[" PLUGIN_NAME "] Strings file closed for reopening\n");
            }
            catch (const std::exception &e)
            {
                _plugin_logprintf("[" PLUGIN_NAME "] Error closing strings file: %s\n", e.what());
            }
        }

        // Открываем файл строк
        try
        {
            stringsFile.open(stringsFilePath, std::ios::out | std::ios::app);
            if (stringsFile.is_open())
            {
                stringsFile << "=== FunCap AUTO-RUN Strings Capture Started at " << getCurrentTime() << " ===" << std::endl;
                stringsFile.flush();
                _plugin_logprintf("[" PLUGIN_NAME "] Strings file REOPENED: %s\n", stringsFilePath.c_str());

                // Добавим тестовую строку для проверки записи
                bool testWrite = writeToStringsFile("Auto-run test string - " + getCurrentTime(), "AUTO-RUN:INIT");
                if (testWrite)
                {
                    _plugin_logprintf("[" PLUGIN_NAME "] Test write to strings file successful\n");
                }
                else
                {
                    _plugin_logprintf("[" PLUGIN_NAME "] Test write to strings file failed\n");
                }
            }
            else
            {
                _plugin_logprintf("[" PLUGIN_NAME "] WARNING: Could not open strings file: %s\n", stringsFilePath.c_str());

                // Попробуем альтернативные пути
                char tempPath[MAX_PATH] = {0};
                GetTempPathA(MAX_PATH, tempPath);
                std::string altPath = std::string(tempPath) + DEFAULT_STRINGS_FILENAME;

                _plugin_logprintf("[" PLUGIN_NAME "] Trying alternative path for strings file: %s\n", altPath.c_str());
                stringsFile.open(altPath, std::ios::out | std::ios::app);

                if (stringsFile.is_open())
                {
                    stringsFilePath = altPath;
                    stringsFile << "=== FunCap AUTO-RUN Strings Capture Started at " << getCurrentTime() << " (Alternative Location) ===" << std::endl;
                    stringsFile.flush();
                    _plugin_logprintf("[" PLUGIN_NAME "] Strings file opened at alternative location: %s\n", stringsFilePath.c_str());

                    // Добавляем тестовую строку в альтернативную локацию
                    bool testWrite = writeToStringsFile("Auto-run test string (alt path) - " + getCurrentTime(), "AUTO-RUN:ALT-PATH");
                    if (testWrite)
                    {
                        _plugin_logprintf("[" PLUGIN_NAME "] Test write to alternative strings file successful\n");
                    }
                    else
                    {
                        _plugin_logprintf("[" PLUGIN_NAME "] Test write to alternative strings file failed\n");
                    }
                }
                else
                {
                    _plugin_logprintf("[" PLUGIN_NAME "] CRITICAL ERROR: Could not open strings file at any location\n");
                }
            }
        }
        catch (const std::exception &e)
        {
            _plugin_logprintf("[" PLUGIN_NAME "] Error opening strings file: %s\n", e.what());
        }

        pluginRunning = true;
    }

    _plugin_logprintf("[" PLUGIN_NAME "] Starting automated run-to-end capture...\n");
    logFile << "=== Automated Run Started at " << getCurrentTime() << " ===" << std::endl;

    // Дополнительный вывод о применении черного списка
    logFile << "=== Using module blacklist with " << blacklistedModules.size() << " entries ===" << std::endl;
    _plugin_logprintf("[" PLUGIN_NAME "] Using module blacklist with %d entries in auto-run mode\n", (int)blacklistedModules.size());

    // Явно убедимся, что ntdll в черном списке
    blacklistedModules.insert("ntdll.dll");
    _plugin_logprintf("[" PLUGIN_NAME "] Ensuring ntdll.dll is in blacklist\n");

    logFile.flush();

    // Удаляем существующие точки останова, чтобы установить новые с учетом черного списка
    _plugin_logprintf("[" PLUGIN_NAME "] Clearing existing breakpoints to respect blacklist\n");
    DbgCmdExec("bc");

    // Добавляем точки останова, учитывая черный список
    _plugin_logprintf("[" PLUGIN_NAME "] Setting breakpoints in non-blacklisted modules only...\n");

    // Получаем текущий исполняемый модуль
    duint cip = DbgValFromString("cip");
    duint mainModuleBase = DbgFunctions()->ModBaseFromAddr(cip);
    char mainModuleName[MAX_MODULE_SIZE] = "";
    if (DbgFunctions()->ModNameFromAddr(mainModuleBase, mainModuleName, true))
    {
        // Проверяем, не находится ли основной модуль в черном списке
        if (isModuleBlacklisted(mainModuleName))
        {
            // Если основной модуль в черном списке, выводим предупреждение и исключаем его из черного списка временно
            _plugin_logprintf("[" PLUGIN_NAME "] WARNING: Main module %s is in blacklist but will be analyzed anyway\n", mainModuleName);
            GuiAddLogMessage("[FunCap] WARNING: Main module is blacklisted. Temporarily allowing breakpoints in this module.\n");

            // Для главного модуля добавляем точки останова независимо от черного списка
            // (иначе не будет обработки основного кода программы)
            std::string moduleNameLower = mainModuleName;
            std::transform(moduleNameLower.begin(), moduleNameLower.end(), moduleNameLower.begin(), ::tolower);

            // Если это не ntdll, то временно удаляем из черного списка
            if (moduleNameLower != "ntdll.dll")
            {
                blacklistedModules.erase(moduleNameLower);
                _plugin_logprintf("[" PLUGIN_NAME "] Temporarily removed %s from blacklist\n", moduleNameLower.c_str());
            }
            else
            {
                _plugin_logprintf("[" PLUGIN_NAME "] Keeping ntdll.dll in blacklist even for main module\n");
            }
        }
    }

    // Добавляем точки останова на функции с учетом черного списка
    menuAddFuncStart();

    // Добавляем точки останова на вызовы функций в текущем модуле с учетом черного списка
    if (mainModuleBase != 0)
    {
        menuAddCallInstructions();

        // Новый код: устанавливаем точки останова на потенциальные строки
        _plugin_logprintf("[" PLUGIN_NAME "] Adding breakpoints for potential string locations...\n");
        GuiAddLogMessage("[FunCap] Searching for potential string locations...\n");

        // Получаем размер модуля
        duint moduleSize = 0;
        if (GetModuleSize(mainModuleBase, &moduleSize))
        {
            // Устанавливаем точки останова на потенциальные строки в этом модуле
            setBreakpointsAtStringLocations(mainModuleBase, moduleSize);
            GuiAddLogMessage("[FunCap] Finished setting breakpoints for potential strings.\n");
        }
        else
        {
            _plugin_logprintf("[" PLUGIN_NAME "] Failed to get module info for string analysis\n");
            GuiAddLogMessage("[FunCap] Warning: Failed to get module info for string analysis.\n");
        }
    }

    // Сохраняем текущее состояние удаления точек останова
    bool originalDeleteBreakpoints = deleteBreakpoints;
    deleteBreakpoints = true; // Включаем удаление точек останова для эффективности

    // Установим флаг автозапуска
    autoRunMode = true;

    _plugin_logprintf("[" PLUGIN_NAME "] Running to end of program in auto-run mode...\n");
    GuiAddLogMessage("[FunCap] Auto-Run started. Running program to completion...\n");

    // Запускаем программу
    DbgCmdExec("run");

    // Чтобы не блокировать интерфейс, мы не ждем здесь завершения
    // Автоматическое продолжение будет обрабатываться в cbBreakpoint

    // Обработчик окончания автозапуска будет вызван из cbDebugEvent,
    // когда программа завершится
}

// Проверяет, находится ли модуль в черном списке
bool isModuleBlacklisted(const char *moduleName)
{
    if (!moduleName || strlen(moduleName) == 0)
        return false;

    // Преобразуем имя модуля в нижний регистр для регистронезависимого сравнения
    std::string moduleNameLower = moduleName;
    std::transform(moduleNameLower.begin(), moduleNameLower.end(), moduleNameLower.begin(), ::tolower);

    return blacklistedModules.find(moduleNameLower) != blacklistedModules.end();
}

// Загружает черный список из файла
void loadBlacklistFromFile()
{
    // Очищаем текущий список
    blacklistedModules.clear();

    // Добавляем стандартные системные модули, которые обычно не интересны для анализа
    blacklistedModules.insert("ntdll.dll");
    blacklistedModules.insert("kernel32.dll");
    blacklistedModules.insert("kernelbase.dll");
    blacklistedModules.insert("user32.dll");
    blacklistedModules.insert("gdi32.dll");
    blacklistedModules.insert("advapi32.dll");
    blacklistedModules.insert("msvcrt.dll");
    blacklistedModules.insert("ws2_32.dll");
    blacklistedModules.insert("wininet.dll");
    blacklistedModules.insert("ole32.dll");
    blacklistedModules.insert("oleaut32.dll");
    blacklistedModules.insert("shell32.dll");
    blacklistedModules.insert("comctl32.dll");
    blacklistedModules.insert("comdlg32.dll");
    blacklistedModules.insert("rpcrt4.dll");
    blacklistedModules.insert("shlwapi.dll");
    blacklistedModules.insert("crypt32.dll");
    blacklistedModules.insert("sechost.dll");
    blacklistedModules.insert("setupapi.dll");
    blacklistedModules.insert("winspool.drv");

    // Добавляем еще больше стандартных Windows DLL
    blacklistedModules.insert("bcrypt.dll");
    blacklistedModules.insert("bcryptprimitives.dll");
    blacklistedModules.insert("clbcatq.dll");
    blacklistedModules.insert("combase.dll");
    blacklistedModules.insert("cryptbase.dll");
    blacklistedModules.insert("cryptsp.dll");
    blacklistedModules.insert("d3d9.dll");
    blacklistedModules.insert("d3d11.dll");
    blacklistedModules.insert("dbghelp.dll");
    blacklistedModules.insert("dhcpcsvc.dll");
    blacklistedModules.insert("dnsapi.dll");
    blacklistedModules.insert("dwmapi.dll");
    blacklistedModules.insert("gdiplus.dll");
    blacklistedModules.insert("imm32.dll");
    blacklistedModules.insert("iphlpapi.dll");
    blacklistedModules.insert("mpr.dll");
    blacklistedModules.insert("msimg32.dll");
    blacklistedModules.insert("mswsock.dll");
    blacklistedModules.insert("ncrypt.dll");
    blacklistedModules.insert("netapi32.dll");
    blacklistedModules.insert("normaliz.dll");
    blacklistedModules.insert("ntmarta.dll");
    blacklistedModules.insert("opengl32.dll");
    blacklistedModules.insert("powrprof.dll");
    blacklistedModules.insert("profapi.dll");
    blacklistedModules.insert("psapi.dll");
    blacklistedModules.insert("schannel.dll");
    blacklistedModules.insert("shcore.dll");
    blacklistedModules.insert("ucrtbase.dll");
    blacklistedModules.insert("urlmon.dll");
    blacklistedModules.insert("uxtheme.dll");
    blacklistedModules.insert("vcruntime140.dll");
    blacklistedModules.insert("vcruntime140d.dll");
    blacklistedModules.insert("version.dll");
    blacklistedModules.insert("winhttp.dll");
    blacklistedModules.insert("winmm.dll");
    blacklistedModules.insert("wintrust.dll");
    blacklistedModules.insert("wsock32.dll");
    blacklistedModules.insert("xinput1_3.dll");

    // Получаем путь к файлу черного списка
    char currentDir[MAX_PATH] = {0};
    GetCurrentDirectoryA(MAX_PATH, currentDir);
    blacklistFilePath = std::string(currentDir) + "\\" + DEFAULT_BLACKLIST_FILENAME;

    // Пытаемся загрузить пользовательский список
    std::ifstream inFile(blacklistFilePath);
    if (inFile.is_open())
    {
        std::string line;
        while (std::getline(inFile, line))
        {
            // Удаляем пробелы и преобразуем в нижний регистр
            std::string trimmedLine = line;
            trimmedLine.erase(0, trimmedLine.find_first_not_of(" \t\r\n"));
            trimmedLine.erase(trimmedLine.find_last_not_of(" \t\r\n") + 1);

            // Игнорируем пустые строки и комментарии
            if (!trimmedLine.empty() && trimmedLine[0] != '#' && trimmedLine[0] != ';')
            {
                std::transform(trimmedLine.begin(), trimmedLine.end(), trimmedLine.begin(), ::tolower);
                blacklistedModules.insert(trimmedLine);
            }
        }
        inFile.close();
    }

    _plugin_logprintf("[" PLUGIN_NAME "] Loaded %d blacklisted modules\n", (int)blacklistedModules.size());
}

// Сохраняет черный список в файл
void saveBlacklistToFile()
{
    std::ofstream outFile(blacklistFilePath);
    if (outFile.is_open())
    {
        outFile << "# FunCap Module Blacklist" << std::endl;
        outFile << "# Add one module name per line (case insensitive)" << std::endl;
        outFile << "# Lines starting with # or ; are comments" << std::endl
                << std::endl;

        for (const std::string &module : blacklistedModules)
        {
            outFile << module << std::endl;
        }
        outFile.close();
        _plugin_logprintf("[" PLUGIN_NAME "] Saved %d blacklisted modules to %s\n",
                          (int)blacklistedModules.size(), blacklistFilePath.c_str());
    }
    else
    {
        _plugin_logprintf("[" PLUGIN_NAME "] Failed to save blacklist to %s\n", blacklistFilePath.c_str());
    }
}

// Функция управления черным списком модулей
void menuManageBlacklist()
{
    _plugin_logprintf("[" PLUGIN_NAME "] Managing module blacklist...\n");

    // Отображаем текущий список модулей
    _plugin_logprintf("[" PLUGIN_NAME "] Current blacklisted modules (%d):\n", (int)blacklistedModules.size());
    for (const std::string &module : blacklistedModules)
    {
        _plugin_logprintf("  %s\n", module.c_str());
    }

    // Запрашиваем у пользователя действие - добавить или удалить модуль
    char cmd[256] = {0};
    DbgCmdExec("ask \"Choose action: 1. Add module to blacklist 2. Remove module from blacklist 3. Reset to default 4. Cancel\"");

    duint choice = DbgValFromString("$result");

    if (choice == 1) // Добавить модуль
    {
        sprintf_s(cmd, "asktext \"Enter module name to blacklist:\"");
        DbgCmdExec(cmd);

        duint textPtr = DbgValFromString("$result");
        if (textPtr)
        {
            char moduleName[MAX_MODULE_SIZE] = {0};
            if (DbgMemRead(textPtr, moduleName, sizeof(moduleName) - 1))
            {
                std::string moduleNameStr = moduleName;
                std::transform(moduleNameStr.begin(), moduleNameStr.end(), moduleNameStr.begin(), ::tolower);

                if (!moduleNameStr.empty())
                {
                    blacklistedModules.insert(moduleNameStr);
                    _plugin_logprintf("[" PLUGIN_NAME "] Added module %s to blacklist\n", moduleNameStr.c_str());
                    GuiAddLogMessage(("[FunCap] Added module " + moduleNameStr + " to blacklist\n").c_str());
                    saveBlacklistToFile();
                }
            }
        }
    }
    else if (choice == 2) // Удалить модуль
    {
        // Создаем список для выбора
        std::string moduleList;
        int index = 1;
        std::map<int, std::string> indexToModule;

        for (const std::string &module : blacklistedModules)
        {
            moduleList += std::to_string(index) + ". " + module + "\\n";
            indexToModule[index] = module;
            index++;
        }

        sprintf_s(cmd, "ask \"Select module to remove from blacklist:\\n%s\"", moduleList.c_str());
        DbgCmdExec(cmd);

        duint selection = DbgValFromString("$result");
        if (selection >= 1 && selection < index)
        {
            std::string moduleToRemove = indexToModule[selection];
            blacklistedModules.erase(moduleToRemove);
            _plugin_logprintf("[" PLUGIN_NAME "] Removed module %s from blacklist\n", moduleToRemove.c_str());
            GuiAddLogMessage(("[FunCap] Removed module " + moduleToRemove + " from blacklist\n").c_str());
            saveBlacklistToFile();
        }
    }
    else if (choice == 3) // Сбросить к значениям по умолчанию
    {
        _plugin_logprintf("[" PLUGIN_NAME "] Resetting blacklist to default values\n");
        blacklistedModules.clear();
        loadBlacklistFromFile();
        saveBlacklistToFile();
        GuiAddLogMessage("[FunCap] Reset module blacklist to default values\n");
    }
    // Для варианта 4 (Отмена) ничего не делаем
}

// Реализация функции для остановки автоматического запуска
void menuPauseAutoRun()
{
    if (!autoRunMode)
    {
        _plugin_logprintf("[" PLUGIN_NAME "] Auto run is not active\n");
        GuiAddLogMessage("[FunCap] Auto run is not currently active.\n");
        return;
    }

    _plugin_logprintf("[" PLUGIN_NAME "] Pausing auto run mode...\n");

    // Проверяем, работает ли программа в данный момент
    if (DbgIsRunning())
    {
        _plugin_logprintf("[" PLUGIN_NAME "] Sending pause command to debuggee\n");
        DbgCmdExec("pause");
    }

    // Отключаем автоматический режим
    autoRunMode = false;

    // Записываем в лог информацию о ручной остановке
    if (logFile.is_open())
    {
        logFile << "=== Auto run manually paused at " << getCurrentTime() << " ===" << std::endl;
        logFile.flush();
    }

    // Записываем в файл строк информацию о ручной остановке
    if (stringsFile.is_open())
    {
        stringsFile << "=== Auto run manually paused at " << getCurrentTime() << " ===" << std::endl;
        stringsFile.flush();
    }

    _plugin_logprintf("[" PLUGIN_NAME "] Auto run mode stopped\n");
    GuiAddLogMessage("[FunCap] Auto run has been paused. You can continue manually or stop debugging.\n");
}

// Добавляем новую функцию для тестирования чтения строк из регистров
void menuTestReadStrings()
{
    _plugin_logprintf("[" PLUGIN_NAME "] Testing read strings from registers...\n");
    GuiAddLogMessage("[FunCap] Testing read strings from registers...\n");

    try
    {
        // Получаем все регистры
        std::vector<RegisterContext> registers = getRegisters();
        _plugin_logprintf("[" PLUGIN_NAME "] Got %d registers\n", (int)registers.size());
        GuiAddLogMessage(("[FunCap] Got " + std::to_string(registers.size()) + " registers\n").c_str());

        // Перебираем все регистры и анализируем их на наличие строк
        int stringsFound = 0;
        for (const auto &reg : registers)
        {
            duint value = reg.value;
            if (value == 0)
                continue;

            // Не пытаемся извлечь строки из регистров управления, IP и системных регистров
            if (reg.name == "EIP" || reg.name == "RIP" ||
                reg.name == "ESP" || reg.name == "RSP" ||
                reg.name == "EBP" || reg.name == "RBP" ||
                reg.name == "EFLAGS" || reg.name == "RFLAGS" ||
                reg.name.find("DR") == 0 || // Debug registers (DR0-DR7)
                reg.name.find("CR") == 0)   // Control registers (CR0-CR8)
                continue;

            // Проверяем, валидный ли указатель
            if (!DbgMemIsValidReadPtr(value))
            {
                _plugin_logprintf("[" PLUGIN_NAME "] Register %s (value: 0x%p) is not a valid pointer\n",
                                  reg.name.c_str(), (void *)value);
                continue;
            }

            // Пытаемся получить строку
            std::string strData = analyzeStringData(value);
            if (!strData.empty() && (strData[0] == '"' || strData.substr(0, 2) == "L\""))
            {
                stringsFound++;

                // Выводим информацию о найденной строке
                std::string logMsg = "String found in " + reg.name + " (0x" +
                                     hexToString(value).substr(2) + "): " + strData;

                _plugin_logprintf("[" PLUGIN_NAME "] %s\n", logMsg.c_str());
                GuiAddLogMessage(("[FunCap] " + logMsg + "\n").c_str());

                // Записываем строку в файл, если открыт
                try
                {
                    std::string cleanStr = strData;
                    if (cleanStr[0] == '"')
                        cleanStr = cleanStr.substr(1, cleanStr.length() - 2);
                    else if (cleanStr.substr(0, 2) == "L\"")
                        cleanStr = cleanStr.substr(2, cleanStr.length() - 3);

                    // Пытаемся записать строку в файл
                    _plugin_logprintf("[" PLUGIN_NAME "] Writing test string to file: %s\n", cleanStr.c_str());
                    bool writeSuccess = writeToStringsFile(cleanStr, "TEST:" + reg.name);

                    if (writeSuccess)
                    {
                        _plugin_logprintf("[" PLUGIN_NAME "] Successfully wrote string to file\n");
                        GuiAddLogMessage("[FunCap] Successfully wrote string to file\n");
                    }
                    else
                    {
                        _plugin_logprintf("[" PLUGIN_NAME "] Failed to write string to file\n");
                        GuiAddLogMessage("[FunCap] Failed to write string to file\n");
                    }
                }
                catch (const std::exception &e)
                {
                    _plugin_logprintf("[" PLUGIN_NAME "] Error writing string to file: %s\n", e.what());
                    GuiAddLogMessage(("[FunCap] Error writing string to file: " + std::string(e.what()) + "\n").c_str());
                }
            }
            else
            {
                _plugin_logprintf("[" PLUGIN_NAME "] No string found in register %s (value: 0x%p)\n",
                                  reg.name.c_str(), (void *)value);
            }
        }

        // Выводим итог
        std::string summaryMsg = "Test completed. Strings found: " + std::to_string(stringsFound);
        _plugin_logprintf("[" PLUGIN_NAME "] %s\n", summaryMsg.c_str());
        GuiAddLogMessage(("[FunCap] " + summaryMsg + "\n").c_str());

        // Если необходимо, проверяем также некоторые диапазоны памяти, где могут быть строки
        duint stackAddr = 0;
        duint baseAddr = 0;
        bool success = false;

        // Получаем адрес стека
        if (isX64)
            stackAddr = DbgEval("rsp", &success);
        else
            stackAddr = DbgEval("esp", &success);

        if (success && DbgMemIsValidReadPtr(stackAddr))
        {
            _plugin_logprintf("[" PLUGIN_NAME "] Testing stack area for strings...\n");
            GuiAddLogMessage("[FunCap] Testing stack area for strings...\n");

            // Проверяем несколько значений на стеке
            for (int i = 0; i < 10; i++)
            {
                duint addrToTest = stackAddr + i * (isX64 ? 8 : 4);
                duint value = 0;

                if (isX64)
                {
                    if (DbgMemRead(addrToTest, &value, 8))
                    {
                        if (DbgMemIsValidReadPtr(value))
                        {
                            std::string strData = analyzeStringData(value);
                            if (!strData.empty() && (strData[0] == '"' || strData.substr(0, 2) == "L\""))
                            {
                                std::string logMsg = "String found on stack at [RSP+" + std::to_string(i * 8) +
                                                     "] (0x" + hexToString(value).substr(2) + "): " + strData;

                                _plugin_logprintf("[" PLUGIN_NAME "] %s\n", logMsg.c_str());
                                GuiAddLogMessage(("[FunCap] " + logMsg + "\n").c_str());

                                // Записываем строку в файл
                                writeToStringsFile(strData[0] == '"' ? strData.substr(1, strData.length() - 2) : strData.substr(2, strData.length() - 3),
                                                   "STACK:RSP+" + std::to_string(i * 8));
                            }
                        }
                    }
                }
                else
                {
                    if (DbgMemRead(addrToTest, &value, 4))
                    {
                        if (DbgMemIsValidReadPtr(value))
                        {
                            std::string strData = analyzeStringData(value);
                            if (!strData.empty() && (strData[0] == '"' || strData.substr(0, 2) == "L\""))
                            {
                                std::string logMsg = "String found on stack at [ESP+" + std::to_string(i * 4) +
                                                     "] (0x" + hexToString(value).substr(2) + "): " + strData;

                                _plugin_logprintf("[" PLUGIN_NAME "] %s\n", logMsg.c_str());
                                GuiAddLogMessage(("[FunCap] " + logMsg + "\n").c_str());

                                // Записываем строку в файл
                                writeToStringsFile(strData[0] == '"' ? strData.substr(1, strData.length() - 2) : strData.substr(2, strData.length() - 3),
                                                   "STACK:ESP+" + std::to_string(i * 4));
                            }
                        }
                    }
                }
            }
        }

        // Проверяем, открыт ли файл строк
        if (stringsFile.is_open())
        {
            _plugin_logprintf("[" PLUGIN_NAME "] Strings file is open: %s\n", stringsFilePath.c_str());
            GuiAddLogMessage(("[FunCap] Strings file is open: " + stringsFilePath + "\n").c_str());
        }
        else
        {
            _plugin_logprintf("[" PLUGIN_NAME "] Strings file is NOT open. Trying to open it.\n");
            GuiAddLogMessage("[FunCap] Strings file is NOT open. Trying to open it.\n");

            // Пробуем открыть файл строк
            stringsFile.open(stringsFilePath, std::ios::out | std::ios::app);
            if (stringsFile.is_open())
            {
                stringsFile << "=== FunCap Test String Read at " << getCurrentTime() << " ===" << std::endl;
                stringsFile.flush();
                _plugin_logprintf("[" PLUGIN_NAME "] Successfully opened strings file: %s\n", stringsFilePath.c_str());
                GuiAddLogMessage(("[FunCap] Successfully opened strings file: " + stringsFilePath + "\n").c_str());
            }
            else
            {
                _plugin_logprintf("[" PLUGIN_NAME "] Failed to open strings file: %s\n", stringsFilePath.c_str());
                GuiAddLogMessage(("[FunCap] Failed to open strings file: " + stringsFilePath + "\n").c_str());
            }
        }
    }
    catch (const std::exception &e)
    {
        _plugin_logprintf("[" PLUGIN_NAME "] Exception in menuTestReadStrings: %s\n", e.what());
        GuiAddLogMessage(("[FunCap] Exception in menuTestReadStrings: " + std::string(e.what()) + "\n").c_str());
    }
    catch (...)
    {
        _plugin_logprintf("[" PLUGIN_NAME "] Unknown exception in menuTestReadStrings\n");
        GuiAddLogMessage("[FunCap] Unknown exception in menuTestReadStrings\n");
    }
}

// Новая функция для определения возможных расположений строк в памяти
bool isPotentialStringLocation(duint addr)
{
    if (!DbgMemIsValidReadPtr(addr))
        return false;

    // Читаем данные по адресу
    std::vector<unsigned char> data = readMemory(addr, 16);
    if (data.empty())
        return false;

    // Проверяем, похожи ли данные на ASCII строку
    int printableCount = 0;
    bool hasZero = false;

    for (size_t i = 0; i < data.size(); i++)
    {
        if (data[i] == 0)
        {
            hasZero = true;
            break;
        }

        if ((data[i] >= 32 && data[i] <= 126) ||
            data[i] == '\r' || data[i] == '\n' || data[i] == '\t')
        {
            printableCount++;
        }
    }

    // Критерии для возможной ASCII строки
    if (printableCount >= 4 && (hasZero || printableCount >= 8))
        return true;

    // Проверяем на UNICODE строку (UTF-16LE)
    int wideCharCount = 0;
    bool hasWideZero = false;

    for (size_t i = 0; i < data.size() - 1; i += 2)
    {
        if (data[i] == 0 && data[i + 1] == 0)
        {
            hasWideZero = true;
            break;
        }

        if (data[i + 1] == 0 &&
            ((data[i] >= 32 && data[i] <= 126) ||
             data[i] == '\r' || data[i] == '\n' || data[i] == '\t'))
        {
            wideCharCount++;
        }
    }

    // Критерии для возможной UNICODE строки
    if (wideCharCount >= 3)
        return true;

    return false;
}

// Функция для установки точек останова в местах потенциальных строк
void setBreakpointsAtStringLocations(duint moduleBase, size_t moduleSize)
{
    _plugin_logprintf("[" PLUGIN_NAME "] Searching for potential string locations in module at 0x%p\n", (void *)moduleBase);

    // Ограничиваем поиск, чтобы не сканировать весь модуль
    const size_t maxScanSize = 1024 * 1024; // 1MB max
    size_t scanSize = (moduleSize > maxScanSize) ? maxScanSize : moduleSize;

    // Шаг сканирования
    const size_t scanStep = 16;
    int potentialStringsFound = 0;
    int skippedBreakpoints = 0;
    const int maxBreakpoints = 50; // Уменьшаем максимальное количество точек останова для лучшей производительности

    for (size_t offset = 0; offset < scanSize && potentialStringsFound < maxBreakpoints; offset += scanStep)
    {
        duint addrToCheck = moduleBase + offset;

        if (isPotentialStringLocation(addrToCheck))
        {
            // Проверяем, не является ли это адрес в секции кода
            // Обычно строковые литералы находятся в секциях данных
            bool isExecutable = false;
            char rights[RIGHTS_STRING_SIZE] = {0};
            if (DbgFunctions()->GetPageRights(addrToCheck, rights))
            {
                isExecutable = (strstr(rights, "Execute") != NULL);
            }

            // Устанавливаем точку останова на доступ чтения (для строковых данных)
            if (!isExecutable && potentialStringsFound < maxBreakpoints) // Ограничиваем количество точек
            {
                // Проверяем, существует ли уже точка останова по этому адресу
                bool breakpointExists = false;
                BPXTYPE bpType = DbgGetBpxTypeAt(addrToCheck);
                if (bpType & bp_memory) // Используем побитовое И для проверки флага bp_memory
                {
                    _plugin_logprintf("[" PLUGIN_NAME "] Skipping existing memory breakpoint at 0x%p\n", (void *)addrToCheck);
                    skippedBreakpoints++;
                    continue;
                }

                char bpCmd[256] = {0};
                sprintf_s(bpCmd, "bpm %p, r", (void *)addrToCheck);

                _plugin_logprintf("[" PLUGIN_NAME "] Setting memory BP for string at 0x%p\n", (void *)addrToCheck);
                DbgCmdExec(bpCmd);
                potentialStringsFound++;

                // Проверяем, успешно ли установлена точка останова
                Sleep(5); // Короткая пауза для применения команды
                bpType = DbgGetBpxTypeAt(addrToCheck);
                if (!(bpType & bp_memory)) {
                    _plugin_logprintf("[" PLUGIN_NAME "] Warning: Failed to set memory breakpoint at 0x%p\n", (void *)addrToCheck);
                    continue;
                }

                // Анализируем и логируем потенциальную строку
                std::string strData = analyzeStringData(addrToCheck);
                if (!strData.empty() && (strData[0] == '"' || strData.substr(0, 2) == "L\""))
                {
                    _plugin_logprintf("[" PLUGIN_NAME "] Potential string found at 0x%p: %s\n",
                                      (void *)addrToCheck, strData.c_str());

                    // Записываем в файл строк
                    writeToStringsFile(strData[0] == '"' ? strData.substr(1, strData.length() - 2) : strData.substr(2, strData.length() - 3),
                                       "AUTO-DETECT:STATIC");
                }
            }
        }
    }

    _plugin_logprintf("[" PLUGIN_NAME "] Finished setting breakpoints for strings. Found %d potential locations, skipped %d existing breakpoints.\n",
                      potentialStringsFound, skippedBreakpoints);
}

// Add a custom function to get module memory information
bool GetModuleSize(duint moduleBase, duint *moduleSize)
{
    *moduleSize = DbgFunctions()->ModSizeFromAddr(moduleBase);
    return (*moduleSize != 0);
}
